<?php

if ( defined( 'WP_LOAD_IMPORTERS' ) ) {
	return;
}

/** Display verbose errors */
define( 'IMPORT_DEBUG', false );

// Load Importer API
require_once ABSPATH . 'wp-admin/includes/import.php';

if ( ! class_exists( 'WP_Importer' ) ) {
	$class_wp_importer = ABSPATH . 'wp-admin/includes/class-wp-importer.php';
	if ( file_exists( $class_wp_importer ) ) {
		require $class_wp_importer;
	}
}

// include WXR file parsers
require CT_THEME_LIB_DIR.'/demo/' . '/parsers.php';


/**
 * Init base stuff
 */

function ct_demo_admin_init() {
	/** @var ctNHP_Options $NHP_OPtions */
	global $NHP_Options;
	add_action(
		'nhp-opts-load-page-' . $NHP_Options->getOptionsPageName(),
		array( 'ctImportDemoHandler', 'optionsPage' )
	);
	add_action( 'wp_ajax_ct-demo-import', array( 'ctImportDemoHandler', 'handleCtDemoImportAction' ) );
	add_action( 'wp_ajax_ct-demo-export', array( 'ctImportDemoHandler', 'handleCtDemoExportAction' ) );
	add_action( 'wp_ajax_ct-demo-rebuild', array( 'ctImportDemoHandler', 'handleCtDemoRebuildAction' ) );
}

// display a success note and hide demo buttons after import
add_filter( 'ct_import.options.demos.show', 'ct_hide_demos_after_successful_import' );
add_filter( 'ct_import_demo.recommends', 'ct_show_successful_demo_alert' );

// echo a success notice
function ct_show_successful_demo_alert( $return ) {
	if ( ! empty( $_GET['ct_1click'] ) ) {
		return '<p class="warning" style="color:green"><strong>' . __( '1-Click Demo Import completed succesfully!' ) . '</strong></p>';
	}
	return $return;
}

// return false to hide import demo buttons
function ct_hide_demos_after_successful_import( $return ) {
	if ( ! empty( $_GET['ct_1click'] ) ) {
		return false;
	}
	return $return;
}

add_action( 'admin_init', 'ct_demo_admin_init', 11 );

add_action( 'ct_theme_loader.options.load', array( 'ctImportDemoHandler', 'addOptions' ) );

require_once CT_THEME_LIB_DIR. '/demo/ctRevolutionSliderImporter.class.php';
require_once CT_THEME_LIB_DIR . '/demo/ctLayerSliderImporter.class.php';
require_once CT_THEME_LIB_DIR . '/demo/ctDemoImagesImporter.class.php';
require_once CT_THEME_LIB_DIR . '/demo/ctWooCommerceImporter.class.php';

add_action( 'ct_import_demo.recommends', array( 'ctRevolutionSliderImporter', 'addRecommendation' ) );
add_action( 'ct_import_demo.recommends', array( 'ctLayerSliderImporter', 'addRecommendation' ) );
add_action( 'ct_import_demo.recommends', array( 'ctWooCommerceImporter', 'addRecommendation' ) );

if ( ! class_exists( 'ctImportDemoHandler' ) ) {

	/**
	 * Class ctImportHandler
	 * 1 click demo import
	 */
	class ctImportDemoHandler {
		/**
		 * Adds option
		 * @return mixed
		 */

		public static function addOptions( $sections ) {

			if ( ! file_exists( CT_THEME_SETTINGS_MAIN_DIR . '/demo' ) ) {
				return $sections;
			}

			$rebuild = isset( $_GET['ct_dev'] );

			$html = '<div style="font-style: normal" class="import">';
			$html .= '<p>' . esc_html__( 'You can setup your Wordpress with just a single click!', 'ct_theme' ) . '</p>';
			$html .= '<p class="hint">' . esc_html__( 'Troubles? Checkout our ', 'ct_theme' ) . ' <a href="http://createit.support/knowledgebase/1-click-demo-import/">FAQ</a></p>';


			$html .= apply_filters( 'ct_import_demo.recommends', '' );

			$html .= '<div class="importDescription">';
			$html .= '<p class="warning" style="color:red"><strong>' . esc_html__(
					'Warning! Import content only on new Wordpress installation. This will ensure your installation will run smoothly.',
					'ct_theme'
				) . '</strong></p>';

			if ( $rebuild || file_exists( CT_THEME_SETTINGS_MAIN_DIR . '/demo/wordpress.xml' ) ) {
				if ( $rebuild ) {
					$html .= '<a href="#" data-action="ct-demo-rebuild" data-dir="" class="ct-demo-import button button-primary">' . esc_html__(
							'REBUILD demo content',
							'ct_theme'
						) . '</a>&nbsp;&nbsp;';
				} else {
					$html .= '<a href="#" data-action="ct-demo-import" data-dir="" class="ct-demo-import button button-primary">' . esc_html__(
							'import demo content',
							'ct_theme'
						) . '</a>&nbsp;&nbsp;';
				}
			}

			//check whether there is additional flavour

			$dirs = glob( CT_THEME_SETTINGS_MAIN_DIR . '/demo/*', GLOB_ONLYDIR );
			if ( is_array( $dirs ) ) {
				do_action( 'ct_import.options.demos.pre', $dirs );
				if ( apply_filters( 'ct_import.options.demos.show', true ) ) {
					//array with: name, logo,buttons(label,dir,class),
					if ( $blocks = apply_filters( 'ct_import.options.demos.blocks', array(), $dirs, $rebuild ) ) {
						foreach ( $blocks as $block ) {
							$html .= '<div class="import-block">';
							if ( isset( $block['logo'] ) ) {
								$html .= '<div class="image"><img src="' . esc_attr( $block['logo'] ) . '"></div>';
							}
							$html .= '<h4>' . $block['name'] . '</h4>';
							$html .= '<div class="buttons">';
							foreach ( $block['buttons'] as $b ) {
								$class = isset( $b['class'] ) ? ' ' . $b['class'] : '';
								$label = isset( $b['label'] ) ? $b['label'] : ucfirst( $b['dir'] );
								$html .= '<a href="#" data-action="ct-demo-' . ( $rebuild ? 'rebuild' : 'import' ) . '" data-dir="' . $b['dir'] . '" class="ct-demo-import button button-primary' . $class . '">' .
								         ( $rebuild ? 'REBUILD ' : '' ) . $label . '</a>';
							}
							$html .= "</div>";

							$html .= '</div>';
						}
					} else {

						foreach ( $dirs as $dir ) {
							$dName = basename( $dir );

							if ( $rebuild ) {
								$html .= '<a href="#" data-action="ct-demo-rebuild" data-dir="' . $dName . '" class="ct-demo-import button button-primary">' . esc_html__(
										'REBUILD demo content',
										'ct_theme'
									) . ': ' . $dName . '</a>&nbsp;&nbsp;';
							} else {
								$html .= '<a href="#" data-action="ct-demo-import" data-dir="' . $dName . '" class="ct-demo-import button button-primary">' . esc_html__(
										'import demo content',
										'ct_theme'
									) . ': ' . $dName . '</a>&nbsp;&nbsp;';
							}
						}
					}
				}
				do_action( 'ct_import.options.demos.post', $dirs );
			}

			$html .= '</div>';
			$html .= '<div class="importLoader" style="display:none">';
			$html .= '<span style="position:relative;top:-28px;padding-right:20px"><strong>' . esc_html__(
					'Importing ...',
					'ct_theme'
				) . '</strong></span><img src="' . CT_THEME_ADMIN_ASSETS_URI . '/images/import_loader.gif" alt="'.esc_html__('loading','ct_theme').'"/>';
			$html .= '</div>';
			$html .= '<div id="ct-demo-importer-init"></div><div id="ct-demo-importer-progressbar"><div id="ct-demo-importer-progresslabel"></div></div><div id="ct-demo-importer-output"></div>';
			$html .= '</div>';

			$section  = array(
				'icon'  => NHP_OPTIONS_URL . 'img/glyphicons/glyphicons_181_download_alt.png',
				'title' => esc_html__( '1-Click Demo Import', 'ct_theme' ),
				'group' => esc_html__( "General", 'ct_theme' ),
				'desc'  => $html,
			);
			$sections = ctNHP_Options::insertSectionAtIndex( $sections, $section, 2 );

			return $sections;
		}

		/**
		 * Load custom JS
		 */

		public static function optionsPage() {
			wp_enqueue_script( 'ct_import_demo',
				CT_THEME_ADMIN_ASSETS_URI . '/js/import.js',
				array( 'jquery', 'jquery-ui-progressbar' ) );
		}

		/**
		 * Import
		 */

		public static function handleCtDemoImportAction() {
			$import = new ctImport();
			$import->dispatch();

			exit;
		}

		/**
		 * Export
		 */

		public static function handleCtDemoExportAction() {
			require_once CT_THEME_LIB_DIR . '/demo/ctDemoExporterImporter.class.php';
			$import = new ctDemoExporterImporter();
			$import->export();

			exit;
		}

		public static function handleCtDemoRebuildAction() {
			require_once CT_THEME_LIB_DIR . '/demo/ctDemoExporterImporter.class.php';
			$import = new ctDemoExporterImporter();
			global $NHP_Options;

			try {
				$import->rebuild( new ctImport(), $NHP_Options );
			} catch ( Exception $e ) {
				exit;
			}
			echo "Rebuild completed successfully";
			exit;
		}
	}
}


/**
 * WordPress Importer class for managing the import process of a WXR file
 *
 * @package WordPress
 * @subpackage Importer
 */

if ( class_exists( 'WP_Importer' ) ) {
	class ctImport extends WP_Importer {
		var $max_wxr_version = 1.2; // max. supported WXR version
		var $id; // WXR attachment ID
		// information to import from WXR file
		var $version;
		var $authors = array();
		var $posts = array();
		var $terms = array();
		var $categories = array();
		var $tags = array();
		var $base_url = '';
		// mappings from old information to new
		var $processed_authors = array();
		var $author_mapping = array();
		var $processed_terms = array();
		var $processed_posts = array();
		var $post_orphans = array();
		var $processed_menu_items = array();
		var $menu_item_orphans = array();
		var $missing_menu_items = array();
		var $fetch_attachments = false;
		var $url_remap = array();
		var $featured_images = array();
		protected $quiet = true;

		function WP_Import() { /* nothing */
			$this->quiet = ! isset( $_GET['ct_dev'] );
		}

		/**
		 * Returns dir where we can find data to import
		 * @return string
		 */

		public static function getDemoContentBaseDir() {
			$dir = CT_THEME_DEMO_DIR;

			if ( isset( $_POST['dir'] ) && $_POST['dir'] ) {
				$dir = $dir . DIRECTORY_SEPARATOR . sanitize_file_name( $_POST['dir'] );
			}

			return $dir;
		}

		/**
		 * Returns XML path
		 * @return string
		 */

		public static function getXmlPath() {
			return self::getDemoContentBaseDir() . '/wordpress.xml';
		}

		/**
		 * Theme options dir
		 * @return string
		 */

		public static function getThemeOptionsPath() {
			return self::getDemoContentBaseDir() . '/theme_options.txt';
		}

		/**
		 * Registered callback function for the WordPress Importer
		 *
		 * Manages the three separate stages of the WXR import process
		 */
		function dispatch() {
			do_action( 'ct_import_pre' );

			//$this->fetch_attachments = ( $this->allow_fetch_attachments() );
			//disable - we use separate importer now
			$this->fetch_attachments = false;

			//import WP settings
			require_once CT_THEME_LIB_DIR . '/demo/ctDemoExporterImporter.class.php';
			$wpImporter = new ctDemoExporterImporter();
			$wpImporter->init(); //hook to data normalization

			if ( $id = $this->import_WXR() ) {
				$file = get_attached_file( $id );

				//import XMLok
				set_time_limit( 0 );
				$this->import( $file );
				wp_delete_attachment( $id );
			}

			$baseDir = self::getDemoContentBaseDir();

			//import theme options
			$options = self::getThemeOptionsPath();
			if ( file_exists( $options ) ) {
				global $NHP_Options;
				//WP_Filesystem();
				//global $wp_filesystem;
				//$NHP_Options->import( $wp_filesystem->get_contents( $options ), $this->processed_posts );
				$NHP_Options->import( file_get_contents( $options ), $this->processed_posts );
			}
			try {

				//import WP settings
				$wpImporter->import( $this->processed_terms, null, $this->processed_posts );

				//import revolution slider
				require_once CT_THEME_LIB_DIR . '/demo/ctRevolutionSliderImporter.class.php';
				$import = new ctRevolutionSliderImporter();
				$import->import();

				//import layer slider
				require_once CT_THEME_LIB_DIR . '/demo/ctLayerSliderImporter.class.php';
				$import = new ctLayerSliderImporter();
				$import->import();
			} catch ( Exception $e ) {
				echo "ERROR: " . $e;
			}

			do_action( 'ct_import_post' );
		}

		/**
		 * Upload a WXR file
		 */
		function import_WXR() {
			$source = self::getXmlPath();

			if ( is_file( $source ) ) {
				$upload_dir = wp_upload_dir();
				$dest       = $upload_dir['path'] . '/' . basename( $source ) . '_.txt';
				copy( $source, $dest );

				//$dest = addslashes($dest);
				$filename = basename( $dest );
				$url      = $upload_dir['url'] . "/$filename";

				if ( is_multisite() ) {
					delete_transient( 'dirsize_cache' );
				}

				// Construct the object array
				$object = array(
					'post_title'     => $filename,
					'post_content'   => $url,
					'post_mime_type' => 'attachment',
					'guid'           => $url
				);

				// Save the data
				return wp_insert_attachment( $object, $dest );
			}
		}

		/**
		 * The main controller for the actual import stage.
		 *
		 * @param string $file Path to the WXR file for importing
		 */
		function import( $file ) {
			add_filter( 'import_post_meta_key', array( $this, 'is_valid_meta_key' ) );
			add_filter( 'http_request_timeout', array( &$this, 'bump_request_timeout' ) );

			$this->import_start( $file );

			$this->get_author_mapping();

			wp_suspend_cache_invalidation( true );
			$this->process_categories();
			$this->process_tags();
			$this->process_terms();
			$this->process_posts();
			wp_suspend_cache_invalidation( false );

			// update incorrect/missing information in the DB
			$this->backfill_parents();
			$this->backfill_attachment_urls();
			$this->remap_featured_images();

			$this->import_end();
		}

		/**
		 * Parses the WXR file and prepares us for the task of processing parsed data
		 *
		 * @param string $file Path to the WXR file for importing
		 */
		function import_start( $file ) {
			if ( ! is_file( $file ) ) {
				echo '<p><strong>' . esc_html__( 'Sorry, there has been an error.', 'ct_theme' ) . '</strong><br/>';
				echo esc_html__( 'The file does not exist, please try again.', 'ct_theme' ) . '</p>';
				$this->footer();
				die();
			}

			$import_data = $this->parse( $file );

			if ( is_wp_error( $import_data ) ) {
				echo '<p><strong>' . esc_html__( 'Sorry, there has been an error.', 'ct_theme' ) . '</strong><br/>';
				echo esc_html( $import_data->get_error_message() ) . '</p>';
				$this->footer();
				die();
			}

			$this->version = $import_data['version'];
			$this->get_authors_from_import( $import_data );
			$this->posts      = $import_data['posts'];
			$this->terms      = $import_data['terms'];
			$this->categories = $import_data['categories'];
			$this->tags       = $import_data['tags'];
			$this->base_url   = esc_url( $import_data['base_url'] );

			wp_defer_term_counting( true );
			wp_defer_comment_counting( true );

			do_action( 'import_start' );
		}

		/**
		 * Performs post-import cleanup of files and the cache
		 */
		function import_end() {
			wp_import_cleanup( $this->id );

			wp_cache_flush();
			foreach ( get_taxonomies() as $tax ) {
				delete_option( "{$tax}_children" );
				_get_term_hierarchy( $tax );
			}

			wp_defer_term_counting( false );
			wp_defer_comment_counting( false );

			do_action( 'import_end' );
		}

		/**
		 * Handles the WXR upload and initial parsing of the file to prepare for
		 * displaying author import options
		 *
		 * @return bool False if error uploading or invalid file, true otherwise
		 */
		function handle_upload() {
			$file = wp_import_handle_upload();

			if ( isset( $file['error'] ) ) {
				echo '<p><strong>' . esc_html__( 'Sorry, there has been an error.', 'ct_theme' ) . '</strong><br/>';
				echo esc_html( $file['error'] ) . '</p>';

				return false;
			} else {
				if ( ! file_exists( $file['file'] ) ) {
					echo '<p><strong>' . esc_html__(
							'Sorry, there has been an error.',
							'ct_theme'
						) . '</strong><br/>';
					printf(
						esc_html__(
							'The export file could not be found at <code>%s</code>. It is likely that this was caused by a
							permissions problem.',
							'ct_theme'
						),
						esc_html( $file['file'] )
					);
					echo '</p>';

					return false;
				}
			}

			$this->id    = (int) $file['id'];
			$import_data = $this->parse( $file['file'] );
			if ( is_wp_error( $import_data ) ) {
				echo '<p><strong>' . esc_html__( 'Sorry, there has been an error.', 'ct_theme' ) . '</strong><br/>';
				echo esc_html( $import_data->get_error_message() ) . '</p>';

				return false;
			}

			$this->version = $import_data['version'];
			if ( $this->version > $this->max_wxr_version ) {
				echo '
			<div class="error"><p><strong>';
				printf(
					esc_html__(
						'This WXR file (version %s) may not be supported by this version of the importer. Please
						consider updating.',
						'ct_theme'
					),
					esc_html( $import_data['version'] )
				);
				echo '</strong></p></div>';
			}

			$this->get_authors_from_import( $import_data );

			return true;
		}

		/**
		 * Retrieve authors from parsed WXR data
		 *
		 * Uses the provided author information from WXR 1.1 files
		 * or extracts info from each post for WXR 1.0 files
		 *
		 * @param array $import_data Data returned by a WXR parser
		 */
		function get_authors_from_import( $import_data ) {
			if ( ! empty( $import_data['authors'] ) ) {
				$this->authors = $import_data['authors'];
				// no author information, grab it from the posts
			} else {
				foreach ( $import_data['posts'] as $post ) {
					$login = sanitize_user( $post['post_author'], true );
					if ( empty( $login ) ) {
						if ( ! $this->quiet ) {

							printf(
								esc_html__(
									'Failed to import author %s. Their posts will be attributed to the current user.',
									'ct_theme'
								),
								esc_html( $post['post_author'] )
							);
							echo '<br/>';
						}
						continue;
					}

					if ( ! isset( $this->authors[ $login ] ) ) {
						$this->authors[ $login ] = array(
							'author_login'        => $login,
							'author_display_name' => $post['post_author']
						);
					}
				}
			}
		}

		/**
		 * Display pre-import options, author importing/mapping and option to
		 * fetch attachments
		 */
		function import_options() {
			$j = 0;
			?>
            <form action="<?php echo admin_url( 'admin.php?import=wordpress&amp;step=2' ); ?>" method="post">
				<?php wp_nonce_field( 'import-wordpress' ); ?>
                <input type="hidden" name="import_id" value="<?php echo esc_attr($this->id); ?>"/>

				<?php if ( ! empty( $this->authors ) ) : ?>
                    <h3><?php esc_html__('Assign Authors', 'ct_theme' ); ?></h3>
                    <p><?php esc_html_e('To make it easier for you to edit and save the imported content, you may want to reassign the author of the imported item to an existing user of this site. For example, you may want to import all the entries as <code>admin</code>s entries.', 'ct_theme'); ?></p>
					<?php if ( $this->allow_create_users() ) : ?>
                        <p><?php printf(
								esc_html__(
									'If a new user is created by WordPress, a new password will be randomly generated and the new user&#8217;s role will be set as %s. Manually changing the new user&#8217;s details will be necessary.',
									'ct_theme'
								),
								esc_html( get_option( 'default_role' ) )
							); ?></p>
					<?php endif; ?>
                    <ol id="authors">
						<?php foreach ( $this->authors as $author ) : ?>
                            <li><?php $this->author_select( $j ++, $author ); ?></li>
						<?php endforeach; ?>
                    </ol>
				<?php endif; ?>

				<?php if ( $this->allow_fetch_attachments() ) : ?>
                    <h3><?php esc_html__('Import Attachments', 'ct_theme' ); ?></h3>
                    <p>
                        <input type="checkbox" value="1" name="fetch_attachments" id="import-attachments"/>
                        <label for="import-attachments"><?php esc_html_e('Download and import file attachments', 'ct_theme'); ?></label>
                    </p>
				<?php endif; ?>

                <p class="submit"><input type="submit" class="button"
                                         value="<?php esc_attr_e( 'Submit', 'ct_theme' ); ?>"/></p>
            </form>
			<?php
		}

		/**
		 * Display import options for an individual author. That is, either create
		 * a new user based on import info or map to an existing user
		 *
		 * @param int $n Index for each author in the form
		 * @param array $author Author information, e.g. login, display name, email
		 */
		function author_select( $n, $author ) {
			esc_html__('Import author:', 'ct_theme' );
			echo ' <strong>' . esc_html( $author['author_display_name'] );
			if ( $this->version != '1.0' ) {
				echo ' (' . esc_html( $author['author_login'] ) . ')';
			}
			echo '</strong><br />';

			if ( $this->version != '1.0' ) {
				echo '<div style="margin-left:18px">';
			}

			$create_users = $this->allow_create_users();
			if ( $create_users ) {
				if ( $this->version != '1.0' ) {
					esc_html__('or create new user with login name:', 'ct_theme' );
					$value = '';
				} else {
					esc_html__('as a new user:', 'ct_theme' );
					$value = esc_attr( sanitize_user( $author['author_login'], true ) );
				}

				echo ' <input type="text" name="user_new[' . esc_attr($n) . ']" value="' . esc_attr($value) . '" /><br />';
			}

			if ( ! $create_users && $this->version == '1.0' ) {
				esc_html__('assign posts to an existing user:', 'ct_theme' );
			} else {
				esc_html__('or assign posts to an existing user:', 'ct_theme' );
			}
			wp_dropdown_users(
				array(
					'name'            => "user_map[$n]",
					'multi'           => true,
					'show_option_all' => esc_html__( '- Select -', 'ct_theme' )
				)
			);
			echo '<input type="hidden" name="imported_authors[' . $n . ']" value="' . esc_attr(
					$author['author_login']
				) . '" />';

			if ( $this->version != '1.0' ) {
				echo '</div>';
			}
		}

		/**
		 * Map old author logins to local user IDs based on decisions made
		 * in import options form. Can map to an existing user, create a new user
		 * or falls back to the current user in case of error with either of the previous
		 */
		function get_author_mapping() {
			if ( ! isset( $_POST['imported_authors'] ) ) {
				return;
			}

			$create_users = $this->allow_create_users();

			foreach ( (array) $_POST['imported_authors'] as $i => $old_login ) {
				// Multisite adds strtolower to sanitize_user. Need to sanitize here to stop breakage in process_posts.
				$santized_old_login = sanitize_user( $old_login, true );
				$old_id             = isset( $this->authors[ $old_login ]['author_id'] ) ? intval(
					$this->authors[ $old_login ]['author_id']
				) : false;

				if ( ! empty( $_POST['user_map'][ $i ] ) ) {
					$user = get_userdata( intval( $_POST['user_map'][ $i ] ) );
					if ( isset( $user->ID ) ) {
						if ( $old_id ) {
							$this->processed_authors[ $old_id ] = $user->ID;
						}
						$this->author_mapping[ $santized_old_login ] = $user->ID;
					}
				} else {
					if ( $create_users ) {
						if ( ! empty( $_POST['user_new'][ $i ] ) ) {
							$user_id = wp_create_user( $_POST['user_new'][ $i ], wp_generate_password() );
						} else {
							if ( $this->version != '1.0' ) {
								$user_data = array(
									'user_login'   => $old_login,
									'user_pass'    => wp_generate_password(),
									'user_email'   => isset( $this->authors[ $old_login ]['author_email'] ) ? $this->authors[ $old_login ]['author_email'] : '',
									'display_name' => $this->authors[ $old_login ]['author_display_name'],
									'first_name'   => isset( $this->authors[ $old_login ]['author_first_name'] ) ? $this->authors[ $old_login ]['author_first_name'] : '',
									'last_name'    => isset( $this->authors[ $old_login ]['author_last_name'] ) ? $this->authors[ $old_login ]['author_last_name'] : '',
								);
								$user_id   = wp_insert_user( $user_data );
							}
						}

						if ( ! is_wp_error( $user_id ) ) {
							if ( $old_id ) {
								$this->processed_authors[ $old_id ] = $user_id;
							}
							$this->author_mapping[ $santized_old_login ] = $user_id;
						} else {
							if ( ! $this->quiet ) {
								printf(
									esc_html__(
										'Failed to create new user for %s. Their posts will be attributed to the current user.',
										'ct_theme'
									),
									esc_html( $this->authors[ $old_login ]['author_display_name'] )
								);
							}
							if ( defined( 'IMPORT_DEBUG' ) && IMPORT_DEBUG ) {
								echo ' ' . esc_html($user_id->get_error_message());
							}
							echo '<br />';
						}
					}
				}

				// failsafe: if the user_id was invalid, default to the current user
				if ( ! isset( $this->author_mapping[ $santized_old_login ] ) ) {
					if ( $old_id ) {
						$this->processed_authors[ $old_id ] = (int) get_current_user_id();
					}
					$this->author_mapping[ $santized_old_login ] = (int) get_current_user_id();
				}
			}
		}

		/**
		 * Create new categories based on import information
		 *
		 * Doesn't create a new category if its slug already exists
		 */
		function process_categories() {
			$this->categories = apply_filters( 'wp_import_categories', $this->categories );

			if ( empty( $this->categories ) ) {
				return;
			}

			foreach ( $this->categories as $cat ) {
				// if the category already exists leave it alone
				$term_id = term_exists( $cat['category_nicename'], 'category' );
				if ( $term_id ) {
					if ( is_array( $term_id ) ) {
						$term_id = $term_id['term_id'];
					}
					if ( isset( $cat['term_id'] ) ) {
						$this->processed_terms[ intval( $cat['term_id'] ) ] = (int) $term_id;
					}
					continue;
				}

				$category_parent      = empty( $cat['category_parent'] ) ? 0 : category_exists(
					$cat['category_parent']
				);
				$category_description = isset( $cat['category_description'] ) ? $cat['category_description'] : '';
				$catarr               = array(
					'category_nicename'    => $cat['category_nicename'],
					'category_parent'      => $category_parent,
					'cat_name'             => $cat['cat_name'],
					'category_description' => $category_description
				);

				$id = wp_insert_category( $catarr );
				if ( ! is_wp_error( $id ) ) {
					if ( isset( $cat['term_id'] ) ) {
						$this->processed_terms[ intval( $cat['term_id'] ) ] = $id;
					}
				} else {
					if ( ! $this->quiet ) {
						printf(
							esc_html__( 'Failed to import category %s', 'ct_theme' ),
							esc_html( $cat['category_nicename'] )
						);

						if ( defined( 'IMPORT_DEBUG' ) && IMPORT_DEBUG ) {
							echo ': ' . esc_html($id->get_error_message());
						}
						echo '<br />';
					}
					continue;
				}
			}

			unset( $this->categories );
		}

		/**
		 * Create new post tags based on import information
		 *
		 * Doesn't create a tag if its slug already exists
		 */
		function process_tags() {
			$this->tags = apply_filters( 'wp_import_tags', $this->tags );

			if ( empty( $this->tags ) ) {
				return;
			}

			foreach ( $this->tags as $tag ) {
				// if the tag already exists leave it alone
				$term_id = term_exists( $tag['tag_slug'], 'post_tag' );
				if ( $term_id ) {
					if ( is_array( $term_id ) ) {
						$term_id = $term_id['term_id'];
					}
					if ( isset( $tag['term_id'] ) ) {
						$this->processed_terms[ intval( $tag['term_id'] ) ] = (int) $term_id;
					}
					continue;
				}

				$tag_desc = isset( $tag['tag_description'] ) ? $tag['tag_description'] : '';
				$tagarr   = array( 'slug' => $tag['tag_slug'], 'description' => $tag_desc );

				$id = wp_insert_term( $tag['tag_name'], 'post_tag', $tagarr );
				if ( ! is_wp_error( $id ) ) {
					if ( isset( $tag['term_id'] ) ) {
						$this->processed_terms[ intval( $tag['term_id'] ) ] = $id['term_id'];
					}
				} else {
					if ( ! $this->quiet ) {
						printf( esc_html__( 'Failed to import post tag %s', 'ct_theme' ),
							esc_html( $tag['tag_name'] ) );
						if ( defined( 'IMPORT_DEBUG' ) && IMPORT_DEBUG ) {
							echo ': ' . esc_html($id->get_error_message());
						}
						echo '<br />';
					}
					continue;
				}
			}

			unset( $this->tags );
		}

		/**
		 * Create new terms based on import information
		 *
		 * Doesn't create a term its slug already exists
		 */
		function process_terms() {
			$this->terms = apply_filters( 'wp_import_terms', $this->terms );

			if ( empty( $this->terms ) ) {
				return;
			}

			foreach ( $this->terms as $term ) {
				// if the term already exists in the correct taxonomy leave it alone
				$term_id = term_exists( $term['slug'], $term['term_taxonomy'] );
				if ( $term_id ) {
					if ( is_array( $term_id ) ) {
						$term_id = $term_id['term_id'];
					}
					if ( isset( $term['term_id'] ) ) {
						$this->processed_terms[ intval( $term['term_id'] ) ] = (int) $term_id;
					}
					continue;
				}

				if ( empty( $term['term_parent'] ) ) {
					$parent = 0;
				} else {
					$parent = term_exists( $term['term_parent'], $term['term_taxonomy'] );
					if ( is_array( $parent ) ) {
						$parent = $parent['term_id'];
					}
				}
				$description = isset( $term['term_description'] ) ? $term['term_description'] : '';
				$termarr     = array(
					'slug'        => $term['slug'],
					'description' => $description,
					'parent'      => intval( $parent )
				);

				$id = wp_insert_term( $term['term_name'], $term['term_taxonomy'], $termarr );
				if ( ! is_wp_error( $id ) ) {
					if ( isset( $term['term_id'] ) ) {
						$this->processed_terms[ intval( $term['term_id'] ) ] = $id['term_id'];
					}
				} else {
					if ( ! $this->quiet && $term['term_taxonomy'] != 'Media' ) {

						printf(
							esc_html__( 'Failed to import %s %s', 'ct_theme' ),
							esc_html( $term['term_taxonomy'] ),
							esc_html( $term['term_name'] )
						);
						if ( defined( 'IMPORT_DEBUG' ) && IMPORT_DEBUG ) {
							echo ': ' . esc_html($id->get_error_message());
						}
						echo '<br />';
					}
					continue;
				}
			}

			unset( $this->terms );
		}

		protected $alreadyExists = array();
		protected $parsing_missing = false;

		/**
		 * Create new posts based on import information
		 *
		 * Posts marked as having a parent which doesn't exist will become top level items.
		 * Doesn't create a new post if: the post type doesn't exist, the given post ID
		 * is already noted as imported or a post with the same title and date already exists.
		 * Note that new/updated terms, comments and meta are imported for the last of the above.
		 */
		function process_posts() {
			$this->posts = apply_filters( 'wp_import_posts', $this->posts );

			foreach ( $this->posts as $post ) {
				$post = apply_filters( 'wp_import_post_data_raw', $post );

				if ( ! post_type_exists( $post['post_type'] ) && ! $this->quiet ) {
					printf(
						esc_html__( 'Failed to import &#8220;%s&#8221;: Invalid post type %s', 'ct_theme' ),
						esc_html( $post['post_title'] ),
						esc_html( $post['post_type'] )
					);
					echo '<br />';
					do_action( 'wp_import_post_exists', $post );
					continue;
				}

				if ( isset( $this->processed_posts[ $post['post_id'] ] ) && ! empty( $post['post_id'] ) ) {
					continue;
				}

				if ( $post['status'] == 'auto-draft' ) {
					continue;
				}

				if ( 'nav_menu_item' == $post['post_type'] ) {
					$this->process_menu_item( $post );
					continue;
				}

				$post_type_object = get_post_type_object( $post['post_type'] );

				$post_exists = post_exists( $post['post_title'], '', $post['post_date'] );
				if ( $post_exists && get_post_type( $post_exists ) == $post['post_type'] ) {
					$this->alreadyExists[ $post_exists ] = true;
					if ( isset($post_type_object->labels->singular_name) && $post_type_object->labels->singular_name != 'Media' ) {
						printf(
							esc_html__( '%s &#8220;%s&#8221; already exists.', 'ct_theme' ),
							$post_type_object->labels->singular_name,
							esc_html( $post['post_title'] )
						);
						echo '<br />';
					}
					$comment_post_ID = $post_id = $post_exists;
				} else {
					$post_parent = (int) $post['post_parent'];
					if ( $post_parent ) {
						// if we already know the parent, map it to the new local ID
						if ( isset( $this->processed_posts[ $post_parent ] ) ) {
							$post_parent = $this->processed_posts[ $post_parent ];
							// otherwise record the parent for later
						} else {
							$this->post_orphans[ intval( $post['post_id'] ) ] = $post_parent;
							$post_parent                                      = 0;
						}
					}

					// map the post author
					$author = sanitize_user( $post['post_author'], true );
					if ( isset( $this->author_mapping[ $author ] ) ) {
						$author = $this->author_mapping[ $author ];
					} else {
						$author = (int) get_current_user_id();
					}

					$postdata = array(
						'import_id'      => $post['post_id'],
						'post_author'    => $author,
						'post_date'      => $post['post_date'],
						'post_date_gmt'  => $post['post_date_gmt'],
						'post_content'   => $post['post_content'],
						'post_excerpt'   => $post['post_excerpt'],
						'post_title'     => $post['post_title'],
						'post_status'    => $post['status'],
						'post_name'      => $post['post_name'],
						'comment_status' => $post['comment_status'],
						'ping_status'    => $post['ping_status'],
						'guid'           => $post['guid'],
						'post_parent'    => $post_parent,
						'menu_order'     => $post['menu_order'],
						'post_type'      => $post['post_type'],
						'post_password'  => $post['post_password']
					);

					$original_post_ID = $post['post_id'];
					$postdata         = apply_filters( 'wp_import_post_data_processed', $postdata, $post );

					if ( 'attachment' == $postdata['post_type'] ) {
						$remote_url = ! empty( $post['attachment_url'] ) ? $post['attachment_url'] : $post['guid'];

						// try to use _wp_attached file for upload folder placement to ensure the same location as the export site
						// e.g. location is 2003/05/image.jpg but the attachment post_date is 2010/09, see media_handle_upload()
						$postdata['upload_date'] = $post['post_date'];
						if ( isset( $post['postmeta'] ) ) {
							foreach ( $post['postmeta'] as $meta ) {
								if ( $meta['key'] == '_wp_attached_file' ) {
									if ( preg_match( '%^[0-9]{4}/[0-9]{2}%', $meta['value'], $matches ) ) {
										$postdata['upload_date'] = $matches[0];
									}
									break;
								}
							}
						}

						$comment_post_ID = $post_id = $this->process_attachment( $postdata, $remote_url );
					} else {
						$comment_post_ID = $post_id = wp_insert_post( $postdata, true );
						do_action( 'wp_import_insert_post', $post_id, $original_post_ID, $postdata, $post );
					}

					if ( is_wp_error( $post_id ) ) {
						if ( $post_type_object->labels->singular_name != 'Media' && ! $this->quiet ) {

							printf(
								esc_html__( 'Failed to import %s &#8220;%s&#8221;', 'ct_theme' ),
								$post_type_object->labels->singular_name,
								esc_html( $post['post_title'] )
							);
							if ( defined( 'IMPORT_DEBUG' ) && IMPORT_DEBUG ) {
								echo ': ' . esc_html($post_id->get_error_message());
							}
							echo '<br />';
						}
						continue;
					}

					if ( $post['is_sticky'] == 1 ) {
						stick_post( $post_id );
					}
				}

				// map pre-import ID to local ID
				$this->processed_posts[ intval( $post['post_id'] ) ] = (int) $post_id;

				if ( ! isset( $post['terms'] ) ) {
					$post['terms'] = array();
				}

				$post['terms'] = apply_filters( 'wp_import_post_terms', $post['terms'], $post_id, $post );

				// add categories, tags and other terms
				if ( ! empty( $post['terms'] ) ) {
					$terms_to_set = array();
					foreach ( $post['terms'] as $term ) {
						// back compat with WXR 1.0 map 'tag' to 'post_tag'
						$taxonomy    = ( 'tag' == $term['domain'] ) ? 'post_tag' : $term['domain'];
						$term_exists = term_exists( $term['slug'], $taxonomy );
						$term_id     = is_array( $term_exists ) ? $term_exists['term_id'] : $term_exists;
						if ( ! $term_id ) {
							$t = wp_insert_term( $term['name'], $taxonomy, array( 'slug' => $term['slug'] ) );
							if ( ! is_wp_error( $t ) ) {
								$term_id = $t['term_id'];
								do_action( 'wp_import_insert_term', $t, $term, $post_id, $post );
							} else {
								if ( $taxonomy != 'Media' && ! $this->quiet ) {
									printf(
										esc_html__( 'Failed to import %s %s', 'ct_theme' ),
										esc_html( $taxonomy ),
										esc_html( $term['name'] )
									);
									if ( defined( 'IMPORT_DEBUG' ) && IMPORT_DEBUG ) {
										echo ': ' . esc_html($t->get_error_message());
									}
									echo '<br />';
								}
								do_action( 'wp_import_insert_term_failed', $t, $term, $post_id, $post );
								continue;
							}
						}
						$terms_to_set[ $taxonomy ][] = intval( $term_id );
					}

					foreach ( $terms_to_set as $tax => $ids ) {
						$tt_ids = wp_set_post_terms( $post_id, $ids, $tax );
						do_action( 'wp_import_set_post_terms', $tt_ids, $ids, $tax, $post_id, $post );
					}
					unset( $post['terms'], $terms_to_set );
				}

				if ( ! isset( $post['comments'] ) ) {
					$post['comments'] = array();
				}

				$post['comments'] = apply_filters( 'wp_import_post_comments', $post['comments'], $post_id, $post );

				// add/update comments
				if ( ! empty( $post['comments'] ) ) {
					$num_comments      = 0;
					$inserted_comments = array();
					foreach ( $post['comments'] as $comment ) {
						$comment_id                                         = $comment['comment_id'];
						$newcomments[ $comment_id ]['comment_post_ID']      = $comment_post_ID;
						$newcomments[ $comment_id ]['comment_author']       = $comment['comment_author'];
						$newcomments[ $comment_id ]['comment_author_email'] = $comment['comment_author_email'];
						$newcomments[ $comment_id ]['comment_author_IP']    = $comment['comment_author_IP'];
						$newcomments[ $comment_id ]['comment_author_url']   = $comment['comment_author_url'];
						$newcomments[ $comment_id ]['comment_date']         = $comment['comment_date'];
						$newcomments[ $comment_id ]['comment_date_gmt']     = $comment['comment_date_gmt'];
						$newcomments[ $comment_id ]['comment_content']      = $comment['comment_content'];
						$newcomments[ $comment_id ]['comment_approved']     = $comment['comment_approved'];
						$newcomments[ $comment_id ]['comment_type']         = $comment['comment_type'];
						$newcomments[ $comment_id ]['comment_parent']       = $comment['comment_parent'];
						$newcomments[ $comment_id ]['commentmeta']          = isset( $comment['commentmeta'] ) ? $comment['commentmeta'] : array();

						if (isset($comment['comment_user_id'])) {
							if (isset($this->processed_authors[$comment['comment_user_id']])) {
								$newcomments[$comment_id]['user_id'] = $this->processed_authors[$comment['comment_user_id']];
							}
						}
					}
					ksort( $newcomments );

					foreach ( $newcomments as $key => $comment ) {
						// if this is a new post we can skip the comment_exists() check
						if ( ! $post_exists || ! comment_exists(
								$comment['comment_author'],
								$comment['comment_date']
							)
						) {
							if ( isset( $inserted_comments[ $comment['comment_parent'] ] ) ) {
								$comment['comment_parent'] = $inserted_comments[ $comment['comment_parent'] ];
							}
							$comment                   = wp_filter_comment( $comment );
							$inserted_comments[ $key ] = wp_insert_comment( $comment );
							do_action(
								'wp_import_insert_comment',
								$inserted_comments[ $key ],
								$comment,
								$comment_post_ID,
								$post
							);

							foreach ( $comment['commentmeta'] as $meta ) {
								$value = maybe_unserialize( $meta['value'] );
								add_comment_meta( $inserted_comments[ $key ], $meta['key'], $value );
							}

							$num_comments ++;
						}
					}
					unset( $newcomments, $inserted_comments, $post['comments'] );
				}

				if ( ! isset( $post['postmeta'] ) ) {
					$post['postmeta'] = array();
				}

				$post['postmeta'] = apply_filters( 'wp_import_post_meta', $post['postmeta'], $post_id, $post );

				// add/update post meta
				if ( ! empty( $post['postmeta'] ) ) {
					foreach ( $post['postmeta'] as $meta ) {
						$key   = apply_filters( 'import_post_meta_key', $meta['key'], $post_id, $post );
						$value = false;

						if ( '_edit_last' == $key ) {
							if ( isset( $this->processed_authors[ intval( $meta['value'] ) ] ) ) {
								$value = $this->processed_authors[ intval( $meta['value'] ) ];
							} else {
								$key = false;
							}
						}

						if ( $key ) {
							// export gets meta straight from the DB so could have a serialized string
							if ( ! $value ) {
								$value = maybe_unserialize( $meta['value'] );
							}

							add_post_meta( $post_id, $key, $value );
							do_action( 'import_post_meta', $post_id, $key, $value );

							// if the post has a featured image, take note of this in case of remap
							if ( '_thumbnail_id' == $key ) {
								$this->featured_images[ $post_id ] = (int) $value;
							}
						}
					}
				}
			}

			unset( $this->posts );
		}

		protected function customMenuItemExists( $item ) {
			$type = '';
			$css  = '';
			$url  = '';

			if ( ! isset( $item['postmeta'] ) ) {
				return false;
			}

			foreach ( $item['postmeta'] as $e ) {
				$key   = $e['key'];
				$value = $e['value'];
				switch ( $key ) {
					case '_menu_item_type':
						$type = $value;
						break;
					case '_menu_item_url':
						$url = $value;
						break;
					case '_menu_item_classes':
						$css = $value;
						break;
				}
			}


			if ( $type == '' || $css == '' || $url == '' ) {
				return false;
			}
			global $wpdb;
			$query   = "SELECT key1.post_id FROM $wpdb->postmeta key1
    INNER JOIN $wpdb->postmeta key2
        ON key1.post_id = key2.post_id
        AND key2.meta_key = '_menu_item_type'
        AND key2.meta_value = %s
    INNER JOIN $wpdb->postmeta key3
        ON key1.post_id = key3.post_id
        AND key3.meta_key = '_menu_item_url'
        AND key3.meta_value = %s
    WHERE key1.meta_key = '_menu_item_classes'
    AND key1.meta_value = %s
";
			$results = $wpdb->get_col( $wpdb->prepare( $query, $type, $url, $css ) );

			return count( $results ) > 0;
		}

		/**
		 * Attempt to create a new menu item from import data
		 *
		 * Fails for draft, orphaned menu items and those without an associated nav_menu
		 * or an invalid nav_menu term. If the post type or term object which the menu item
		 * represents doesn't exist then the menu item will not be imported (waits until the
		 * end of the import to retry again before discarding).
		 *
		 * @param array $item Menu item details from WXR file
		 * @param bool $parsing_missing
		 */
		function process_menu_item( $item ) {
			// skip draft, orphaned menu items
			if ( 'draft' == $item['status'] ) {
				return;
			}

			//createIT - custom check
			//check if this item already exists. Disabled for now
			/*if ( $this->customMenuItemExists( $item ) ) {
				return;
			}

			if ( ! $this->parsing_missing && ( ! isset( $item['postmeta']['_menu_item_type'] ) || isset( $item['postmeta']['_menu_item_type'] ) != 'custom' ) ) {
				$this->missing_menu_items[] = $item;

				return;
			}*/

			$menu_slug = false;
			if ( isset( $item['terms'] ) ) {
				// loop through terms, assume first nav_menu term is correct menu
				foreach ( $item['terms'] as $term ) {
					if ( 'nav_menu' == $term['domain'] ) {
						$menu_slug = $term['slug'];
						break;
					}
				}
			}

			// no nav_menu term associated with this menu item
			if ( ! $menu_slug ) {
				esc_html__('Menu item skipped due to missing menu slug', 'ct_theme' );
				echo '<br />';

				return;
			}

			$menu_id = term_exists( $menu_slug, 'nav_menu' );
			if ( ! $menu_id ) {
				printf(
					esc_html__( 'Menu item skipped due to invalid menu slug: %s', 'ct_theme' ),
					esc_html( esc_html($menu_slug) )
				);
				echo '<br />';

				return;
			} else {
				$menu_id = is_array( $menu_id ) ? $menu_id['term_id'] : $menu_id;
			}

			foreach ( $item['postmeta'] as $meta ) {
				${$meta['key']} = $meta['value'];
			}

			if ( 'taxonomy' == $_menu_item_type && isset( $this->processed_terms[ intval( $_menu_item_object_id ) ] ) ) {
				$_menu_item_object_id = $this->processed_terms[ intval( $_menu_item_object_id ) ];
			} else {
				if ( 'post_type' == $_menu_item_type && isset( $this->processed_posts[ intval(
							$_menu_item_object_id
						) ] )
				) {

					//createIT - we have to check whether this item wasn't already imported
					if ( $this->parsing_missing && isset( $this->alreadyExists[ intval( $_menu_item_object_id ) ] ) ) {
						return;
					}

					$_menu_item_object_id = $this->processed_posts[ intval( $_menu_item_object_id ) ];
				} else {
					if ( 'custom' != $_menu_item_type ) {
						// associated object is missing or not imported yet, we'll retry later
						$this->missing_menu_items[] = $item;

						return;
					}
				}
			}

			if ( isset( $this->processed_menu_items[ intval( $_menu_item_menu_item_parent ) ] ) ) {
				$_menu_item_menu_item_parent = $this->processed_menu_items[ intval( $_menu_item_menu_item_parent ) ];
			} else {
				if ( $_menu_item_menu_item_parent ) {
					$this->menu_item_orphans[ intval( $item['post_id'] ) ] = (int) $_menu_item_menu_item_parent;
					$_menu_item_menu_item_parent                           = 0;
				}
			}

			// wp_update_nav_menu_item expects CSS classes as a space separated string
			$_menu_item_classes = maybe_unserialize( $_menu_item_classes );
			if ( is_array( $_menu_item_classes ) ) {
				$_menu_item_classes = implode( ' ', $_menu_item_classes );
			}

			$args = array(
				'menu-item-object-id'   => $_menu_item_object_id,
				'menu-item-object'      => $_menu_item_object,
				'menu-item-parent-id'   => $_menu_item_menu_item_parent,
				'menu-item-position'    => intval( $item['menu_order'] ),
				'menu-item-type'        => $_menu_item_type,
				'menu-item-title'       => $item['post_title'],
				'menu-item-url'         => $_menu_item_url,
				'menu-item-description' => $item['post_content'],
				'menu-item-attr-title'  => $item['post_excerpt'],
				'menu-item-target'      => $_menu_item_target,
				'menu-item-classes'     => $_menu_item_classes,
				'menu-item-xfn'         => $_menu_item_xfn,
				'menu-item-status'      => $item['status']
			);

			$id = wp_update_nav_menu_item( $menu_id, 0, $args );
			if ( $id && ! is_wp_error( $id ) ) {
				$this->processed_menu_items[ intval( $item['post_id'] ) ] = (int) $id;
			}
		}

		/**
		 * If fetching attachments is enabled then attempt to create a new attachment
		 *
		 * @param array $post Attachment post details from WXR
		 * @param string $url URL to fetch attachment from
		 *
		 * @return int|WP_Error Post ID on success, WP_Error otherwise
		 */
		function process_attachment( $post, $url ) {
			if ( ! $this->fetch_attachments ) {
				return new WP_Error(
					'attachment_processing_error',
					esc_html__( 'Fetching attachments is not enabled', 'ct_theme' )
				);
			}

			// if the URL is absolute, but does not contain address, then upload it assuming base_site_url
			if ( preg_match( '|^/[\w\W]+$|', $url ) ) {
				$url = rtrim( $this->base_url, '/' ) . $url;
			}

			$upload = $this->fetch_remote_file( $url, $post );
			if ( is_wp_error( $upload ) ) {
				return $upload;
			}

			if ( $info = wp_check_filetype( $upload['file'] ) ) {
				$post['post_mime_type'] = $info['type'];
			} else {
				return new WP_Error( 'attachment_processing_error', esc_html__( 'Invalid file type', 'ct_theme' ) );
			}

			$post['guid'] = $upload['url'];

			// as per wp-admin/includes/upload.php
			$post_id = wp_insert_attachment( $post, $upload['file'] );
			wp_update_attachment_metadata( $post_id, wp_generate_attachment_metadata( $post_id, $upload['file'] ) );

			// remap resized image URLs, works by stripping the extension and remapping the URL stub.
			if ( preg_match( '!^image/!', $info['type'] ) ) {
				$parts = pathinfo( $url );
				$name  = basename( $parts['basename'], ".{$parts['extension']}" ); // PATHINFO_FILENAME in PHP 5.2

				$parts_new = pathinfo( $upload['url'] );
				$name_new  = basename( $parts_new['basename'], ".{$parts_new['extension']}" );

				$this->url_remap[ $parts['dirname'] . '/' . $name ] = $parts_new['dirname'] . '/' . $name_new;
			}

			return $post_id;
		}

		/**
		 * Attempt to download a remote file attachment
		 *
		 * @param string $url URL of item to fetch
		 * @param array $post Attachment details
		 *
		 * @return array|WP_Error Local file location details on success, WP_Error otherwise
		 */
		function fetch_remote_file( $url, $post ) {
			// extract the file name and extension from the url
			$file_name = basename( $url );

			// get placeholder file in the upload dir with a unique, sanitized filename
			$upload = wp_upload_bits( $file_name, 0, '', $post['upload_date'] );
			if ( $upload['error'] ) {
				return new WP_Error( 'upload_dir_error', $upload['error'] );
			}

			// fetch the remote url and write it to the placeholder file
			//$headers = wp_get_http( $url, $upload['file'] );
			$result = wp_remote_get($url, array(
				'stream' => true,
				'filename' => $upload['file']
			));

			// request failed
			if ($result instanceof WP_Error) {
				@unlink( $upload['file'] );

				return new WP_Error( 'import_file_error', esc_html__( 'Remote server did not respond', 'ct_theme' ) );
			}


			// make sure the fetch was successful
			if ( $result['response']['code'] != '200' ) {
				@unlink( $upload['file'] );

				return new WP_Error(
					'import_file_error',
					sprintf(
						esc_html__( 'Remote server returned error response %1$d %2$s', 'ct_theme' ),
						esc_html( $result['body'] ),
						get_status_header_desc( $result['response']['code'] )
					)
				);
			}

			$filesize = filesize( $upload['file'] );

			if ( isset( $result['response']['code'] ) && $filesize != $result['response']['code'] ) {
				@unlink( $upload['file'] );

				return new WP_Error( 'import_file_error', esc_html__( 'Remote file is incorrect size', 'ct_theme' ) );
			}

			if ( 0 == $filesize ) {
				@unlink( $upload['file'] );

				return new WP_Error( 'import_file_error', esc_html__( 'Zero size file downloaded', 'ct_theme' ) );
			}

			$max_size = (int) $this->max_attachment_size();
			if ( ! empty( $max_size ) && $filesize > $max_size ) {
				@unlink( $upload['file'] );

				return new WP_Error(
					'import_file_error',
					sprintf(
						esc_html__( 'Remote file is too large, limit is %s', 'ct_theme' ),
						size_format( $max_size )
					)
				);
			}

			// keep track of the old and new urls so we can substitute them later
			$this->url_remap[ $url ]          = $upload['url'];
			$this->url_remap[ $post['guid'] ] = $upload['url']; // r13735, really needed?
			// keep track of the destination if the remote url is redirected somewhere else
			if ( isset( $headers['x-final-location'] ) && $headers['x-final-location'] != $url ) {
				$this->url_remap[ $headers['x-final-location'] ] = $upload['url'];
			}

			return $upload;
		}

		/**
		 * Attempt to associate posts and menu items with previously missing parents
		 *
		 * An imported post's parent may not have been imported when it was first created
		 * so try again. Similarly for child menu items and menu items which were missing
		 * the object (e.g. post) they represent in the menu
		 */
		function backfill_parents() {
			global $wpdb;

			// find parents for post orphans
			foreach ( $this->post_orphans as $child_id => $parent_id ) {
				$local_child_id = $local_parent_id = false;
				if ( isset( $this->processed_posts[ $child_id ] ) ) {
					$local_child_id = $this->processed_posts[ $child_id ];
				}
				if ( isset( $this->processed_posts[ $parent_id ] ) ) {
					$local_parent_id = $this->processed_posts[ $parent_id ];
				}

				if ( $local_child_id && $local_parent_id ) {
					$wpdb->update(
						$wpdb->posts,
						array( 'post_parent' => $local_parent_id ),
						array( 'ID' => $local_child_id ),
						'%d',
						'%d'
					);
				}
			}

			// all other posts/terms are imported, retry menu items with missing associated object
			$missing_menu_items = $this->missing_menu_items;
			//createIT set we are looking for missing stuff
			$this->parsing_missing = true;
			foreach ( $missing_menu_items as $item ) {
				$this->process_menu_item( $item );
			}

			// find parents for menu item orphans
			foreach ( $this->menu_item_orphans as $child_id => $parent_id ) {
				$local_child_id = $local_parent_id = 0;
				if ( isset( $this->processed_menu_items[ $child_id ] ) ) {
					$local_child_id = $this->processed_menu_items[ $child_id ];
				}
				if ( isset( $this->processed_menu_items[ $parent_id ] ) ) {
					$local_parent_id = $this->processed_menu_items[ $parent_id ];
				}

				if ( $local_child_id && $local_parent_id ) {
					update_post_meta( $local_child_id, '_menu_item_menu_item_parent', (int) $local_parent_id );
				}
			}
		}

		/**
		 * Use stored mapping information to update old attachment URLs
		 */
		function backfill_attachment_urls() {
			global $wpdb;
			// make sure we do the longest urls first, in case one is a substring of another
			uksort( $this->url_remap, array( &$this, 'cmpr_strlen' ) );

			foreach ( $this->url_remap as $from_url => $to_url ) {
				// remap urls in post_content
				$wpdb->query(
					$wpdb->prepare(
						"UPDATE {$wpdb->posts} SET post_content = REPLACE(post_content, %s, %s)",
						$from_url,
						$to_url
					)
				);
				// remap enclosure urls
				$result = $wpdb->query(
					$wpdb->prepare(
						"UPDATE {$wpdb->postmeta} SET meta_value = REPLACE(meta_value, %s, %s) WHERE meta_key='enclosure'",
						$from_url,
						$to_url
					)
				);
			}
		}

		/**
		 * Update _thumbnail_id meta to new, imported attachment IDs
		 */
		function remap_featured_images() {
			// cycle through posts that have a featured image
			foreach ( $this->featured_images as $post_id => $value ) {
				if ( isset( $this->processed_posts[ $value ] ) ) {
					$new_id = $this->processed_posts[ $value ];
					// only update if there's a difference
					if ( $new_id != $value ) {
						update_post_meta( $post_id, '_thumbnail_id', $new_id );
					}
				}
			}
		}

		/**
		 * Parse a WXR file
		 *
		 * @param string $file Path to WXR file for parsing
		 *
		 * @return array Information gathered from the WXR file
		 */
		function parse( $file ) {
			$parser = new WXR_Parser();

			return $parser->parse( $file );
		}

		// Display import page title
		function header() {
			echo '<div class="wrap">';
			echo '<h2>' . esc_html__( 'Import WordPress', 'ct_theme' ) . '</h2>';

			$updates  = get_plugin_updates();
			$basename = plugin_basename(CT_THEME_LIB_DIR.'/demo/install.php');
			if ( isset( $updates[ $basename ] ) ) {
				$update = $updates[ $basename ];
				echo '<div class="error"><p><strong>';
				printf(
					esc_html__(
						'A new version of this importer is available. Please update to version %s to ensure compatibility with newer export files.',
						'ct_theme'
					),
					$update->update->new_version
				);
				echo '</strong></p></div>';
			}
		}

		// Close div.wrap
		function footer() {
			echo '</div>';
		}

		/**
		 * Display introductory text and file upload form
		 */
		function greet() {
			echo '<div class="narrow">';
			echo '<p>' . esc_html__(
					'Howdy! Upload your WordPress eXtended RSS (WXR) file and we&#8217;ll import the posts, pages, comments, custom fields, categories, and tags into this site.',
					'ct_theme'
				) . '</p>';
			echo '<p>' . esc_html__(
					'Choose a WXR (.xml) file to upload, then click Upload file and import.',
					'ct_theme'
				) . '</p>';
			wp_import_upload_form( 'admin.php?import=wordpress&amp;step=1' );
			echo '</div>';
		}

		/**
		 * Decide if the given meta key maps to information we will want to import
		 *
		 * @param string $key The meta key to check
		 *
		 * @return string|bool The key if we do want to import, false if not
		 */
		function is_valid_meta_key( $key ) {
			// skip attachment metadata since we'll regenerate it from scratch
			// skip _edit_lock as not relevant for import
			if ( in_array( $key, array( '_wp_attached_file', '_wp_attachment_metadata', '_edit_lock' ) ) ) {
				return false;
			}

			return $key;
		}

		/**
		 * Decide whether or not the importer is allowed to create users.
		 * Default is true, can be filtered via import_allow_create_users
		 *
		 * @return bool True if creating users is allowed
		 */
		function allow_create_users() {
			return apply_filters( 'import_allow_create_users', true );
		}

		/**
		 * Decide whether or not the importer should attempt to download attachment files.
		 * Default is true, can be filtered via import_allow_fetch_attachments. The choice
		 * made at the import options screen must also be true, false here hides that checkbox.
		 *
		 * @return bool True if downloading attachments is allowed
		 */
		function allow_fetch_attachments() {
			return apply_filters( 'import_allow_fetch_attachments', true );
		}

		/**
		 * Decide what the maximum file size for downloaded attachments is.
		 * Default is 0 (unlimited), can be filtered via import_attachment_size_limit
		 *
		 * @return int Maximum attachment file size to import
		 */
		function max_attachment_size() {
			return apply_filters( 'import_attachment_size_limit', 0 );
		}

		/**
		 * Added to http_request_timeout filter to force timeout at 60 seconds during import
		 * @return int 60
		 */
		function bump_request_timeout( $val ) {
			return 60;
		}

		// return the difference in length between two strings
		function cmpr_strlen( $a, $b ) {
			return strlen( $b ) - strlen( $a );
		}
	}
} // class_exists( 'WP_Importer' )