<?php

/**
 * modify the default query for Portofio Loop (sets post per page value
 * @param $query
 */
function ct_pre_get_posts($query)
{

    //gets options from customizer
    $itemsOrderBy = ct_get_context_option('portfolio_index_items_order_by', 'name');
    $itemsOrder = ct_get_context_option('portfolio_index_items_order', 'ASC');


    if (is_post_type_archive('portfolio') && !is_admin() && $query->is_main_query()) {
        if (ct_get_context_option('portfolio_index_show_pagination', 1) == '1') {
            $query->set('posts_per_page', ct_get_context_option('portfolio_index_limit', 10));
            $query->set('orderby', $itemsOrderBy);
            $query->set('order', $itemsOrder);
        } else {
            $query->set('posts_per_page', -1);
            $query->set('orderby', $itemsOrderBy);
            $query->set('order', $itemsOrder);
        }
    }
}

add_action('pre_get_posts', 'ct_pre_get_posts');


/**
 *
 * returns Gallery category array by customizer
 * @return array|WP_Error
 *
 */
function ct_get_gallery_terms_customized()
{
    $catsCustomOrder = ct_get_context_option('portfolio_index_cats_custom_order', '');


    if ($catsCustomOrder) {
        $catsOrderBy = false;
        $catsOrder = false;
    } else {
        $catsOrderBy = ct_get_context_option('portfolio_index_cats_order_by', 'name');
        $catsOrder = ct_get_context_option('portfolio_index_cats_order', 'ASC');
    }


    if ($catsCustomOrder !== '') {
        $taxonomy = 'portfolio_category';
        $catsCustomOrder = explode(',', $catsCustomOrder);
        $terms = array();
        foreach ($catsCustomOrder as $termSlug) {
            $termObj = get_term_by('slug', $termSlug, $taxonomy);
            if (is_object($termObj)) {
                $terms[] = $termObj;
            }
        }
    } else {
        $taxonomy = 'portfolio_category';
        $term_args = array(
            'hide_empty' => true,
            'orderby' => esc_sql($catsOrderBy),
            'order' => esc_sql($catsOrder)
        );
        $terms = get_terms($taxonomy, $term_args);
    }

    return $terms;

}


/**
 * Returns category buttons HTML
 * @param array $cats
 * @return string
 */
function ct_portfolio_get_cats($cats = array())
{

    if (ct_get_context_option('portfolio_index_show_cats', '0') == '0') {
        return '';
    }

    $showAll = ct_get_context_option('portfolio_index_show_cats_all', '1') == '1' ? true : false;
    $allLabel = $showAll ? ct_get_context_option('portfolio_index_cats_all_label', 'All') : '';
    $terms = ct_get_gallery_terms_customized();

    if (!$terms) {
        return '';
    }


    $buttonRaw = ' <a href="%url%" class="btn btn-lg btn-button--dark">%title%</a>';

    $output = '';
    foreach ($terms as $term) {
        if (is_object($term)){
            $output .= str_replace(
                array('%title%', '%url%'),
                //category url fix
                array($term->name, esc_url(add_query_arg(array('portfolio_category'=>$term->slug),esc_url(CT_CURRENT_URL)))),
                $buttonRaw);
        }

    }

    if ($showAll) {
        $output = str_replace(array('%title%', '%url%'), array($allLabel, get_post_type_archive_link('portfolio')), $buttonRaw) . $output;
    }


    return $output;
}

add_filter('wp_title', 'ct_set_gallery_title',100);
function ct_set_gallery_title($orig_title) {
	global $post;
    if (!is_object($post)){
        return'';
    }
	$post_type = $post->post_type;

	if ( $post_type == 'portfolio' ) {
		return esc_html__('Gallery','ct_theme');
	} else {
		return $orig_title;
	}
}
