<?php




// add custom class to tag - macaroon
/**
 * @param $html
 * @return mixed
 */
function ct_add_class_the_tags($html)
{
    $html = str_replace('<a', '<a class="ct-blogTag"', $html);
    return $html;
}

add_filter('the_tags', 'ct_add_class_the_tags');


//macaroon
/**
 * @param $html
 * @return mixed
 */
function ct_custom_blog_avatar_class($html)
{
    $html = str_replace('class=\'', 'class=\'img-circle ', $html);
    return $html;
}

add_filter('get_avatar', 'ct_custom_blog_avatar_class');



if (!function_exists('ct_esc_title')) {
    /**
     * @return string
     */
    function ct_esc_title($title)
    {
        $title = wp_kses($title, array (
            'b' => array(),
            'i' => array(),
        ));

        return $title;
    }
}


if (!function_exists('ct_get_author_fullname')) {
    /**
     * @return string
     */
    function ct_get_author_fullname()
    {
        $fname = get_the_author_meta('first_name');
        $lname = get_the_author_meta('last_name');

        if (empty($fname)) {
            $full_name = $lname;
        } elseif (empty($lname)) {
            $full_name = $fname;
        } else {
            //both first name and last name are present
            $full_name = $fname . ' ' . $lname;
        }
        return $full_name;
    }
}


if (!function_exists('ct_theme_setup_posts')) {
    /**
     * Set basic formats etc
     */

    function ct_theme_setup_posts()
    {

        //custom theme formats
        add_theme_support('post-formats', array('aside', 'gallery', 'link', 'image', 'quote', 'video', 'audio'));

        //add size for featured image
        add_image_size('featured_image', 558, 312, true);
        //add_image_size('gallery_thumb', 558, 312, true);
        add_image_size('gallery_thumb', 251, 188, true);

        add_image_size('gallery_thumb_infinite_1', 400, 640, true);
        add_image_size('gallery_thumb_infinite_2', 400, 250, true);

        add_image_size('ct_portfolio_slider_thumb', 172, 172, true);
    }
}
add_action('after_setup_theme', 'ct_theme_setup_posts');


if (!function_exists('ct_get_post_short_month_name')) {
    /**
     * @return string
     */
    function ct_get_post_short_month_name()
    {
        $time = mktime(0, 0, 0, get_the_time('m'));
        $short_month_name = strftime("%b", $time);
        return $short_month_name;
    }
}


if (!function_exists('ct_get_post_day')) {
    /**
     * @return int|string
     */
    function ct_get_post_day()
    {
        $day = get_the_time('d');

        return $day;
    }
}


/**
 * returns video html for video format post
 */
if (!function_exists('ct_post_video')) {

    /**
     * @param $postid
     * @param int $width
     * @param int $height
     * @return string
     */
    function ct_post_video($postid, $width = 500, $height = 300)
    {
        $m4v = get_post_meta($postid, 'videoM4V', true);
        $ogv = get_post_meta($postid, 'videoOGV', true);
        $direct = get_post_meta($postid, 'videoDirect', true);
        return (do_shortcode('[video width="' . $width . '" height="' . $height . '" link="' . $direct . '" m4v="' . $m4v . '" ogv="' . $ogv . '"]'));
    }
}


/**
 * returns audio html for audio format post
 */
if (!function_exists('ct_post_audio')) {
    /**
     * @param $postid
     * @param int $width
     * @param int $height
     */
    function ct_post_audio($postid, $width = 500, $height = 300)
    {
        $mp3 = get_post_meta($postid, 'audioMP3', true);
        $ogg = get_post_meta($postid, 'audioOGA', true);
        $poster = get_post_meta($postid, 'audioPoster', true);
        $audioPosterHeight = get_post_meta($postid, 'audioPosterHeight', true);

        if (!$mp3 && !$ogg){
            return'';
        }
        return do_shortcode('[audio height="' . $height . '" width="' . $width . '" mp3="' . $mp3 . '" ogg="' . $ogg . '" poster="' . $poster . '" posterheight="' . $audioPosterHeight . '"]');
    }
}


if (!function_exists('get_section_class')) {
    /**
     * @param string $type
     *
     * @return string
     */
    function get_section_class($type = 'blog')
    {
        $classes = array();

        switch ($type) {

            case 'blog-index':
            default:
                $clases[] = 'ct-u-paddingTop100';
                $clases[] = 'ct-u-paddingBottom50';
                break;

            case 'portfolio-standard':
                $clases[] = 'ct-u-paddingTop100';
                $clases[] = 'ct-u-paddingBottom50';
                break;

            case 'portfolio-ajax':
                $clases[] = 'ct-u-paddingTop100';
                $clases[] = 'ct-u-paddingBottom50';
                break;

            case 'faq':
                $clases[] = 'ct-u-paddingTop100';
                $clases[] = 'ct-u-paddingBottom70';
                $clases[] = 'ct-u-backgroundGray';
                break;

            case 'post-single':
                $clases[] = 'ct-u-paddingBoth100';
                $clases[] = 'ct-u-paddingBottom50';
                $clases[] = 'ct-u-backgroundGray';
                $clases[] = ct_get_context_option('general_flavour') == 'ct--darkMotive' ? 'ct-u-backgroundDarkGray' : '';
                break;


            case 'single-portfolio-standard':
                $clases[] = 'ct-u-paddingTop100';
                $clases[] = 'ct-u-paddingBottom50';
                break;

            case 'single-portfolio-magnificPopup':
            case 'single-portfolio-ajax':
                $clases[] = 'ct-gallerySingleAjax';
                break;


            case '404':
                $clases[] = 'ct-u-paddingBoth150';
                $clases[] = 'ct-u-backgroundGray';
                break;

            case 'author':
                $clases[] = 'ct-u-paddingTop100';
                $clases[] = 'ct-u-paddingBottom150';
                $clases[] = 'ct-u-backgroundGray2';
                break;

            case 'category':
                $clases[] = 'ct-u-paddingTop100';
                $clases[] = 'ct-u-paddingBottom150';
                $clases[] = 'ct-u-backgroundGray2';
                break;

            case 'image':
                $clases[] = 'ct-u-paddingTop100';
                $clases[] = 'ct-u-paddingBottom150';
                $clases[] = 'ct-u-backgroundGray2';
                break;

            case 'page-left':
                $clases[] = 'ct-u-paddingTop100';
                $clases[] = 'ct-u-paddingBottom150';
                $clases[] = 'ct-u-backgroundGray2';
                break;

            case 'page-both':
                $clases[] = 'ct-u-paddingTop100';
                $clases[] = 'ct-u-paddingBottom150';
                $clases[] = 'ct-u-backgroundGray2';
                break;

            case 'page-right':
                $clases[] = 'ct-u-paddingTop100';
                $clases[] = 'ct-u-paddingBottom150';
                $clases[] = 'ct-u-backgroundGray2';
                break;

            case 'search':
                $clases[] = 'ct-u-paddingTop100';
                $clases[] = 'ct-u-paddingBottom150';
                $clases[] = 'ct-u-backgroundGray2';
                break;

        }

        return implode(' ', $classes);
    }
}


if (!function_exists('ct_update_post_counter')) {
    /**
     * @param string $id
     */
    function ct_update_post_counter($id = '')
    {
        /*update view count meta*/
        $view_count = get_post_meta($id, 'post_view_count', true);
        if ($view_count == '') {
            $view_count = 0;
            delete_post_meta($id, 'post_view_count');
            add_post_meta($id, 'post_view_count', 0);
        } else {
            $view_count++;
            update_post_meta($id, 'post_view_count', $view_count);
        }
    }
}


if (!function_exists('ct_get_blog_item_title')) {
    /**
     * @return string
     */
    function ct_get_blog_item_title()
    {
        $postTitle = get_the_title();
        $custom = get_post_custom(get_the_ID());


        if (!isset($custom['show_title'][0])) {
            $custom['show_title'][0] = 'global';
        }

        if (isset($custom['show_title'][0])) {
            if (ct_get_context_option("posts_single_show_title", 1) && $custom['show_title'][0] == 'global') {
                return $postTitle;

            } else {
                return '';
            }
        }

        if (isset($custom['show_title'][0]) && $custom['show_title'][0] == 'yes') {
            return $postTitle;
        }

        return '';
    }
}


if (!function_exists('ct_new_excerpt_more')) {
    /**
     * @param $more
     *
     * @return string
     */
    function ct_new_excerpt_more($more)
    {
        return '';
    }
}
add_filter('ct_excerpt_more', 'new_excerpt_more');


/* Custom Excerpt length */
if (!function_exists('ct_new_excerpt_length')) {
    /**
     * @return mixed|void
     */
    function ct_new_excerpt_length()
    {
        return (ct_get_context_option("posts_show_index_as", 'content') == 'content') ? apply_filters('ct_excerpt_length_content', 60)
            : apply_filters('ct_excerpt_length', 20);
    }
}
add_filter('excerpt_length', 'ct_new_excerpt_length', 999);


if (!function_exists('ct_comments_count')) {
    /**
     *
     */
    function ct_comments_count()
    {
        comments_number(__('No Comments', 'ct_theme'), '1 ' . __('Comment', 'ct_theme'), '% ' . __('Comments', 'ct_theme'));
    }
}

if (!function_exists('ct_get_excerpt_by_id')) {
    /**
     *
     * @param $post_id
     * @param int $lenght
     * @param string $after_text
     *
     * @return string
     */
    function ct_get_excerpt_by_id($post_id, $lenght = 10, $after_text = '&#8230;')
    {
        $the_post = get_post($post_id);
        $the_excerpt = $the_post->post_excerpt;
        if ('' !==$the_excerpt) {
            return '<p>' . strip_tags(strip_shortcodes($the_excerpt)) . '</p>';
        }else{
            $the_excerpt = $the_post->post_content;
        }

        $extended = get_extended($the_excerpt);

        //adds #more-postid to link
        $ismore = @strpos($the_post->post_content, '<!--more-->');
        if ($ismore) {
            add_filter('ct_more_id', 'ct_add_more_id');
            return do_shortcode($extended['main']);
        }

        the_content();
        return '';
    }
}



if (!function_exists('ct_get_excerpt_stripped_by_id')) {
    /**
     *
     * @param $post_id
     * @param int $lenght
     * @param string $after_text
     *
     * @return string
     */
    function ct_get_excerpt_stripped_by_id($post_id, $lenght = 10, $after_text = '&#8230;')
    {
        $the_post = get_post($post_id);


        $the_excerpt = $the_post->post_excerpt;
        if (!$the_excerpt) {
            $the_excerpt = $the_post->post_content;
        }
        $excerpt_length = $lenght;

        $extended = get_extended($the_excerpt);

	    //adds #more-postid to link
        $ismore = @strpos( $the_post->post_content, '<!--more-->');
        if (  $ismore){
	        add_filter('ct_more_id', 'ct_add_more_id');
            return $extended['main'];
        }

	    $the_excerpt = strip_tags(strip_shortcodes($the_excerpt));
	    $words = explode(' ', $the_excerpt, $excerpt_length + 1);
	    if (count($words) > $excerpt_length) {
            array_pop($words);
            array_push($words, $after_text);
            $the_excerpt = implode(' ', $words);
        }

        $the_excerpt = '<p>' . $the_excerpt . '</p>';

        return $the_excerpt;
    }
}


if (!function_exists('ct_add_more_id')) {
    /**
     * @param $str
     * @return string
     */
    function ct_add_more_id($str)
    {
        global $post;
        if (is_object($post)) {
            return '#more-' . $post->ID;
        } else {
            return '';
        }
    }
}


if (!function_exists('ct_get_next_post')) {
    /**
     * @return array|null
     */
    function ct_get_next_post($imageSize = 'featured_image')
    {
        $next_post = get_next_post();
        if (is_object($next_post)) {
            return array(
                'image' => ct_get_feature_image_src($next_post->ID, $imageSize),
                'title' => $next_post->post_title,
                'excerpt' => ct_get_excerpt_by_id($next_post->ID, 6, '')
            );
        }
        return '';
    }
}


if (!function_exists('ct_get_previous_post')) {
    /**
     * @return array|null
     */
    function ct_get_previous_post($imageSize = 'featured_image')
    {
        $previous_post = get_previous_post();

        if (is_object($previous_post)) {
            return array(
                'image' => ct_get_feature_image_src($previous_post->ID, $imageSize),
                'title' => $previous_post->post_title,
                'excerpt' => ct_get_excerpt_by_id($previous_post->ID, 6, '')
            );
        }
        return '';
    }
}


if (!function_exists('ct_get_current_commenter_url')) {

    /**
     * @return string
     */
    function ct_get_current_commenter_url()
    {
        $commentID = get_comment_ID();
        $obj = get_comment($commentID);
        $id = $obj->user_id;
        return get_author_posts_url($id);
    }
}

//modify avatar <img> class
add_filter('get_avatar','change_avatar_css');

function change_avatar_css($class) {
    $class = str_replace("class='img-circle", "class='pull-left img-circle", $class) ;
    return $class;
}

