<?php

/**
 * Alert Box shortcode
 */
class ctAlertBoxShortcode extends ctShortcode implements ctVisualComposerShortcodeInterface{

    /**
     * Returns name
     * @return string|void
     */
    public function getName() {
        return 'Alert Box';
    }

    /**
     * Shortcode name
     * @return string
     */
    public function getShortcodeName() {
        return 'alert_box';
    }

    /**
     * Shortcode type
     * @return string
     */
    public function getShortcodeType() {
        return self::TYPE_SHORTCODE_ENCLOSING;
    }


    /**
     * Handles shortcode
     * @param $atts
     * @param null $content
     * @return string
     */
    public function handle($atts, $content = null) {
        extract(shortcode_atts($this->extractShortcodeAttributes($atts), $atts));

        $mainContainerAtts = array(
            'class' => array(
                'alert',
                $type,
                $class
            ),
            'id' => $id,
            'role'=>'alert'
        );

        //default
        $iconClass = 'fa fa-check-circle-o';

        switch ($type) {
            case 'danger':
                $iconClass = 'fa fa-ban';
                break;
            case 'info':
                $iconClass = 'fa fa-info-circle';
                break;
            case 'warning':
                $iconClass = 'fa fa-exclamation-circle';
                break;
        }
        $buttonHtml = '<!--Button Start-->
                        <button type="button" class="close" data-dismiss="alert" aria-hidden="true">&times;</button>
                        <!--Button End-->';
        $html = '<!--Alert Start-->
                <div ' . $this->buildContainerAttributes($mainContainerAtts, $atts) . '>
                    ' . $buttonHtml .'
                    '. $content . '
                </div>
                <!--Audio End-->';
        return do_shortcode($html);

        ?>

<?php
    }

    /**
     * Returns config
     * @return null
     */
    public function getAttributes() {
        return array(
            'type' => array(
	            'label' => __('type', 'ct_theme'),
	            'default' => 'alert-info',
	            'type' => "select",
	            'help' => __( "Select type of the box color", 'ct_theme' ),
                'choices' => array(
                    'alert-danger' => __('danger', 'ct_theme'),
                    'alert-success' => __('success', 'ct_theme'),
                    'alert-info' => __('info', 'ct_theme'),
                    'alert-warning' => __('warning', 'ct_theme'),
                )
            ),



            'content' => array('label' => __('Message', 'ct_theme'), 'default' => '', 'type' => 'textarea'),
            'id' => array('type' => false, 'default' => ''),
            'class' => array('label' => __("Custom CSS class", 'ct_theme'))
        );
    }

    /**
     * Returns additional info about VC
     * @return ctVisualComposerInfo
     */
    public function getVisualComposerInfo() {
        return new ctVisualComposerInfo( $this, array(
	        'icon' => 'fa-exclamation-triangle',
	        'description' => __( "Custom alert box", 'ct_theme')
	        ) );
    }
}

new ctAlertBoxShortcode();