<?php

/**
 * infobox shortcode
 */
class ctInfoBoxShortcode extends ctShortcode implements ctVisualComposerShortcodeInterface
{

    /**
     * Returns name
     * @return string|void
     */
    public function getName()
    {
        return 'Info box';
    }

    /**
     * Shortcode name
     * @return string
     */
    public function getShortcodeName()
    {
        return 'info_box';
    }

    /**
     * Returns shortcode type
     * @return mixed|string
     */

    public function getShortcodeType()
    {
        return self::TYPE_SHORTCODE_ENCLOSING;
    }


    /**
     * Handles shortcode
     *
     * @param $atts
     * @param null $content
     *
     * @return string
     */

    public function handle($atts, $content = null)
    {
        extract(shortcode_atts($this->extractShortcodeAttributes($atts), $atts));

        $buttonRel = $this->buildRelatedShortcode('button', $atts, 'button_rel');
        $dividerRel = $this->buildRelatedShortcode('ct_divider', $atts, 'divider_rel');


        $mainContainerAtts = array(
            'class' => array(
                'ct-frame',
                'ct-frame--white',
                $class,
            ),
        );


        $html = '<!--Info Box Start-->
                    <section class="ct-frame ct-frame--white">
                        <h3 class="ct-u-font1 text-uppercase text-center">
                            ' . $header . '
                        </h3>
                        ' . $dividerRel . '
                        <p>
                            ' . $content . '
                        </p>
                        <div class="text-center ct-u-paddingBoth20">
                            ' . $buttonRel . '
                        </div>
                    </section>
                <!--Info Box End-->';

        return $html;
    }


    /**
     * Returns config
     * @return null
     */
    public function getAttributes()
    {
        $items = array();
        $items['header'] = array('label' => __("Box header", 'ct_theme'),
            'default' => '',
            'type' => 'input',
        );

        $items['content'] = array('label' => __('Content', 'ct_theme'), 'default' => '', 'type' => "textarea");


        $items['class'] = array('label' => __("Custom class", 'ct_theme'),
            'default' => '',
            'type' => 'input',
            'help' => __('Adding custom class allows you to set diverse styles in css to the element. Type in name of class, which you defined in css. You can add as much classes as you like.', 'ct_theme')
        );

        //add related header attributes
        $items = $this->mergeShortcodeAttributes($items, 'button', 'button_rel', __('Button', 'ct_theme'));
        $items = $this->mergeShortcodeAttributes($items, 'ct_divider', 'divider_rel', __('Divider', 'ct_theme'));
        return $items;
    }


    /**
     * Returns additional info about VC
     * @return ctVisualComposerInfo
     */
    public function getVisualComposerInfo()
    {
        return new ctVisualComposerInfo($this, array(
                'icon' => 'fa-external-link-square',
                'container' => false
            )
        );
    }
}

new ctInfoBoxShortcode();



