<?php

/**
 * Image shortcode
 */
class ctPromoBoxShortcode extends ctShortcode implements ctVisualComposerShortcodeInterface
{


    /**
     * Returns name
     * @return string|void
     */
    public function getName()
    {
        return 'Promo Box';
    }

    /**
     * Shortcode name
     * @return string
     */
    public function getShortcodeName()
    {
        return 'promo_box';
    }


    public function enqueueScripts()
    {
        wp_register_script('ct_isotope', CT_THEME_ASSETS . '/js/portfolio/jquery.isotope.min.js', array('jquery'), null, true);
        wp_enqueue_script('ct_isotope');

        wp_register_script('ct_infinitescroll', CT_THEME_ASSETS . '/js/portfolio/infinitescroll.min.js', array('jquery'), null, true);
        wp_enqueue_script('ct_infinitescroll');

        wp_register_script('ct_imagesloaded', CT_THEME_ASSETS . '/js/portfolio/imagesloaded.js', array('jquery'), null, true);
        wp_enqueue_script('ct_imagesloaded');

        wp_register_script('ct_init', CT_THEME_ASSETS . '/js/portfolio/init.js', array('jquery'), null, true);
        wp_enqueue_script('ct_init');

        wp_register_script('ct_magnific_popup', CT_THEME_ASSETS . '/plugins/magnific-popup/jquery.magnific-popup.min.js', array('jquery'), null, true);
        wp_enqueue_script('ct_magnific_popup');

        wp_register_script('ct_imgbox_init', CT_THEME_ASSETS . '/js/image-box/init.js', array('jquery'), null, true);
        wp_enqueue_script('ct_imgbox_init');

        wp_register_style('ct-magnific-popup-style', CT_THEME_ASSETS . '/plugins/magnific-popup/jplayer.css');
        wp_enqueue_style('ct-magnific-popup-style');
    }


    /**
     * Handles shortcode
     * @param $atts
     * @param null $content
     * @return string
     */

    public function handle($atts, $content = null)
    {

        extract(shortcode_atts($this->extractShortcodeAttributes($atts), $atts));
        $dividerRel = $this->buildRelatedShortcode('ct_divider', $atts, 'divider_rel');



        $mainContainerAtts = array(
            'class' => array(
                $class,
                'ct-imageBox'
            ),
        );

        $bright1 = $bright_text == 'yes' ? 'ct-u-colorWhite' : 'ct-u-colorMotive';
        $bright2 = $bright_text == 'yes' ? 'ct-u-colorWhite' : '';


        $html = '<!--Promo Box Start-->
                    <div' . $this->buildContainerAttributes($mainContainerAtts, $atts) . '>
                        <a href="' . $link . '">
                            ' . ct_get_image($src, $alt) . '
                            <article class="ct-imageBox-inner">
                                <div class="ct-imageBox-innerAlign">
                                    <div class="row">
                                        <div class="' . ($center_header == 'no' ? 'col-sm-6 col-sm-offset-6 text-center' : 'col-sm-12 text-center') . '">
                                            ' . $dividerRel . '
                                            <h5 class="' . $bright1 . ' ct-u-font3 text-uppercase">
                                                ' . $title . '
                                            </h5>
                                            <h3 class="' . $bright2 . ' ct-u-font2 text-uppercase">
                                                ' . $subtitle . '
                                            </h3>
                                        </div>
                                    </div>
                                </div>
                            </article>
                        </a>
                    </div>
                <!--Promo Box End-->';
        return do_shortcode($html);
    }

    /**
     * Returns config
     * @return null
     */
    public function getAttributes()
    {
        $items = array(
            'widgetmode' => array('default' => 'false', 'type' => false),
            'src' => array('label' => __("image", 'ct_theme'), 'default' => '', 'type' => 'image', 'help' => __("Image source", 'ct_theme')),
            'alt' => array('label' => __('alt', 'ct_theme'), 'default' => ' ', 'type' => 'input', 'help' => __("Alternate text", 'ct_theme')),
            'title' => array('label' => __('Title', 'ct_theme'), 'default' => '', 'type' => 'input'),
            'subtitle' => array('label' => __('Subtitle', 'ct_theme'), 'default' => '', 'type' => 'input'),
            'link' => array('label' => __('link', 'ct_theme'), 'help' => __("ex. http://www.google.com", 'ct_theme')),
            'bright_text' => array(
                'label' => __('Bright text', 'ct_theme'),
                'default' => 'no',
                'type' => "select",
                'choices' => array(
                    'no' => __('no', 'ct_theme'),
                    'yes' => __('yes', 'ct_theme'),
                )
            ),
            'center_header' => array(
                'label' => __('Header centered', 'ct_theme'),
                'default' => 'no',
                'type' => "select",
                'choices' => array(
                    'no' => __('no', 'ct_theme'),
                    'yes' => __('yes', 'ct_theme'),
                )
            ),
            'class' => array('label' => __("Custom class", 'ct_theme'), 'default' => '', 'type' => 'input', 'help' => __('Adding custom class allows you to set diverse styles in css to the element. Type in name of class, which you defined in css. You can add as much classes as you like.', 'ct_theme')),
        );


        $items = $this->mergeShortcodeAttributes($items, 'ct_divider', 'divider_rel', __('Setup Divider', 'ct_theme'));
        return $items;
    }


    /**
     * Returns additional info about VC
     * @return ctVisualComposerInfo
     */
    public function getVisualComposerInfo()
    {
        return new ctVisualComposerInfo($this, array(
            'icon' => 'fa-picture-o',
            'description' => __("Add an image", 'ct_theme')
        ));
    }
}

new ctPromoBoxShortcode();