<?php

/**
 * Button shortcode
 */
class ctButtonShortcode extends ctShortcode implements ctVisualComposerShortcodeInterface
{


    /**
     * Returns name
     * @return string|void
     */
    public function getName()
    {
        return 'Button';
    }

    /**
     * Shortcode name
     * @return string
     */
    public function getShortcodeName()
    {
        return 'button';
    }

    /**
     * Shortcode type
     * @return string
     */
    public function getShortcodeType()
    {
        return self::TYPE_SHORTCODE_ENCLOSING;
    }


    /**
     * Handles shortcode
     * @param $atts
     * @param null $content
     * @return string
     */

    public function handle($atts, $content = null)
    {
        extract(shortcode_atts($this->extractShortcodeAttributes($atts), $atts));
        $iconHtml = $this->buildRelatedShortcode('icon', $atts, 'icon_rel');

        //check if related icon exist
        if (!isset($atts['icon_rel_name'])) {
            $iconHtml = '';
        }elseif($atts['icon_rel_name'] ===''){
            $iconHtml = '';
        }


        if ($autolink == 'blog') {
            $link = get_permalink(get_option('page_for_posts'));
        }

        if ($autolink == 'portfolio') {
            $link = get_post_type_archive_link('portfolio');
        }

        $mainContainerAtts = array(
            'class' => array(
                'btn',
                $class,
                $style,
                $disabled,
                $size,
                $block,
                $animated=='yes'?'animated':''
            ),
            'href' => esc_url($link)
        );
        if ($id) {
            $mainContainerAtts['id'] = $id;
        }

        if ($animated=='yes') {
            $mainContainerAtts['data-fx'] = 'fadeIn';
            $mainContainerAtts['data-hover'] = $hover_text;
        }

        if ($iconHtml){
            $ButtonHtml = '<a  ' . $this->buildContainerAttributes($mainContainerAtts, $atts) . '>' . $iconHtml. ' ' . $button_text . '</a>';
        }elseif ($style =='btn-button--brown' || $style =='btn-button--dark'){
            $ButtonHtml = '<a' . $this->buildContainerAttributes($mainContainerAtts, $atts) . '>' . $button_text . '</a>';
        }else{

            $ButtonHtml = '<a' . $this->buildContainerAttributes($mainContainerAtts, $atts) . '><span>' . $button_text . '</span></a>';
        }


        return do_shortcode('<!--Button Start-->'
                                .$ButtonHtml.
                            '<!--Button End-->');
    }

    /**
     * Returns config
     * @return null
     */
    public function getAttributes()
    {
        $items = array(
            'button_text' => array('label' => __('Text', 'ct_theme'), 'default' => '', 'type' => 'input'),
            'id' => array('default' => false, 'type' => false),
            'size' => array('label' => __('Size', 'ct_theme'), 'default' => 'normal', 'type' => 'select', 'choices' =>
                array(
                    'btn-xs' => __('very small', 'ct_theme'),
                    'btn-sm' => __('small', 'ct_theme'),
                    'btn-lg' => __('large', 'ct_theme'),
                )
            , 'help' => __("Button size", 'ct_theme')),

            'block' => array('label' => __('Block?', 'ct_theme'), 'default' => '', 'type' => 'select', 'choices' =>
                array(
                    '' => __('no', 'ct_theme'),
                    'btn-block' => __('yes', 'ct_theme'),
                ),
            ),

            'style' => array('label' => __('Style', 'ct_theme'), 'default' => 'btn-button--dark', 'type' => 'select', 'choices' =>
                array(
                    'btn-button--dark' => __('default', 'ct_theme'),
                    'btn-default' => __('white', 'ct_theme'),
                    'btn-inverse' => __('inverse', 'ct_theme'),
                    'btn-button--brown' => __('motive', 'ct_theme'),

                    'btn-primary' => __('primary', 'ct_theme'),
                    'btn-success' => __('success', 'ct_theme'),
                    'btn-danger' => __('danger', 'ct_theme'),
                    'btn-warning' => __('warning', 'ct_theme'),
                    'btn-info' => __('info', 'ct_theme'),
                ),
                'help' => __("Button style", 'ct_theme')
            ),
            'disabled' => array(
                'label' => __("Disabled", 'ct_theme'),
                'type' => 'checkbox',
                'default' => '',
                'value' => 'disabled',
            ),

            'autolink' => array('label' => __('Autolink', 'ct_theme'), 'default' => '', 'type' => 'select', 'choices' =>
                array(
                    'none' => __('none', 'ct_theme'),
                    'blog' => __('blog index', 'ct_theme'),
                    'portfolio' => __('portfolio', 'ct_theme'),
                ),
            ),

            'link' => array('label' => __('Link', 'ct_theme'), 'help' => __("ex. http://www.google.com", 'ct_theme'), 'dependency' => array(
                'element' => 'autolink',
                'value' => array('none')
            )),


            ///////
            'animated' => array('label' => __('Animated', 'ct_theme'), 'default' => '', 'type' => 'select', 'choices' =>
                array(
                    'no' => __('no', 'ct_theme'),
                    'yes' => __('yes', 'ct_theme'),
                ),
            ),

            'hover_text' => array('label' => __('Hover text', 'ct_theme'), 'dependency' => array(
                'element' => 'animated',
                'value' => array('yes')
            )),


            'class' => array('label' => __("Custom class", 'ct_theme'), 'default' => '', 'type' => 'input', 'help' => __('Adding custom class allows you to set diverse styles in css to the element. Type in name of class, which you defined in css. You can add as much classes as you like.', 'ct_theme')),
        );
        return $this->mergeShortcodeAttributes($items, 'icon', 'icon_rel', __('Icon', 'ct_theme'));
    }

    /**
     * Returns additional info about VC
     * @return ctVisualComposerInfo
     */
    public function getVisualComposerInfo()
    {
        return new ctVisualComposerInfo($this, array(
            'icon' => 'fa-plus-square',
        ));
    }
}

new ctButtonShortcode();