<?php

/**
 * Google maps shortcode
 */
class ctGoogleMapsShortcode extends ctShortcode implements ctVisualComposerShortcodeInterface
{

    /**
     * Returns name
     * @return string|void
     */
    public function getName()
    {
        return 'Google maps';
    }

    /**
     * Shortcode name
     * @return string
     */
    public function getShortcodeName()
    {
        return 'google_maps';
    }

    /**
     * Enqueue scripts
     */

    public function enqueueScripts()
    {
        wp_register_script('ct_google_maps_api', 'https://maps.googleapis.com/maps/api/js?v=3.exp&sensor=false', array('jquery'), false, true);
        wp_enqueue_script('ct_google_maps_api');

        wp_register_script('ct-gmap', CT_THEME_ASSETS . '/js/gmaps/gmap3.min.js', array('ct_google_maps_api'), false, true);
        wp_enqueue_script('ct-gmap');

        wp_register_script('ct-gmap-init', CT_THEME_ASSETS . '/js/gmaps/init.js', array('ct-gmap'), false, true);
        wp_enqueue_script('ct-gmap-init');
    }


    /**
     * Handles shortcode
     *
     * @param $atts
     * @param null $content
     *
     * @return string
     */

    public function handle($atts, $content = null)
    {
        $attributes = shortcode_atts($this->extractShortcodeAttributes($atts), $atts);
        extract($attributes);

        $id = ($id == '') ? 'gmap' . rand(100, 1000) : $id;


        if (!is_numeric($height)) {
            $height = '100%';
        }

        if ($attributes['map_draggable'] == 'yes') {
            $attributes['map_draggable'] = 'true';
        } else if ($attributes['map_draggable'] == 'no') {
            $attributes['map_draggable'] = 'false';
        }


        if (ct_is_browser_type('mobile') == true) {
            $attributes['map_draggable'] = 'false';
        }

        $mainContainerAtts = array(
            'class' => array(
                'ct-googleMap',
                'ct-googlemapStyle'
            ),
            'data-height' => $height,
            'data-offset' => $offset,
            'data-location' => $location,
            'data-map_draggable' => $attributes['map_draggable'],
            'data-map_type' => $attributes['map_type'],
            'data-zoom' => $zoom,
            'data-marker' => $marker,
            'id' => $id,
            'data-longitude' => $lon,
            'data-latitude' => $lat,
        );

        $titleShortcode = $title ? '[header class="special" level="3"]' . $title . '[/header]' : '';



            $html = do_shortcode('<!--Map Start-->'
                                    . $titleShortcode . '
                                    <div ' . $this->buildContainerAttributes($mainContainerAtts, $atts) . '></div>
                                <!--Map End-->');



        return $html;

    }


    /**
     * Returns config
     * @return null
     */
    public function getAttributes()
    {
        return array(
            'id' => array(
                'label' => __('ID', 'ct_theme'),
                'default' => '',
                'type' => 'input',
                'help' => __("Custom map ID", 'ct_theme')
            ),
            'location' => array(
                'label' => __('Location', 'ct_theme'),
                'default' => '',
                'type' => 'input',
                'help' => __("Enter location eg town", 'ct_theme')
            ),
            'lat' => array(
                'label' => __('Latitude', 'ct_theme'),
                'default' => '',
                'type' => 'input',
                'help' => __("Enter latitude", 'ct_theme')
            ),
            'lon' => array(
                'label' => __('longitude', 'ct_theme'),
                'default' => '',
                'type' => 'input',
                'help' => __("Enter longitude", 'ct_theme')
            ),
            'height' => array('label' => __('height', 'ct_theme'), 'default' => '', 'type' => 'input'),
            'offset' => array(
                'label' => __('Map vertical offset (in px)', 'ct_theme'),
                'default' => '0',
                'type' => 'input'
            ),
            'title' => array(
                'label' => __('Title', 'ct_theme'),
                'default' => '',
                'type' => 'input',
                'help' => __("Title", 'ct_theme')
            ),
            'map_type' => array(
                'label' => __('Select map type', 'ct_theme'),
                'default' => 'ROADMAP',
                'type' => 'select',
                'options' => array(
                    'ROADMAP' => 'Roadmap',
                    'SATELLITE' => 'Satellite',
                    'HYBRID' => 'Hybrid',
                    'TERRAIN' => 'Terrain',
                )
            ),
            'marker' => array('label' => __('Marker icon', 'ct_theme'), 'default' => CT_THEME_ASSETS.'/images/marker.png', 'type' => 'image', 'help' => __("Icon which will be used as marker. Leave empty for default.", 'ct_theme')),
            'map_draggable' => array(
                'label' => __('Draggable', 'ct_theme'),
                'default' => 'true',
                'type' => 'select',
                'options' => array('true' => 'true', 'false' => 'false'),
                'help' => __("locked automatically on mobile devices", 'ct_theme')
            ),
            'zoom' => array(
                'label' => __("zoom", 'ct_theme'),
                'type' => 'input',
                'default' => 15,
                'help' => __('Zoom from 1 to 15', 'ct_theme')
            ),
            'roll' => array(
                'label' => __('Roll Map', 'ct_theme'),
                'default' => 'no',
                'type' => 'select',
                'choices' => array(
                    'yes' => __('yes', 'ct_theme'),
                    'no' => __('no', 'ct_theme')
                ),
            ),
            'expanded_label' => array(
                'label' => __('Expanded map label', 'ct_theme'),
                'default' => 'hide map',
                'type' => 'input',
            ),
            'collapsed_label' => array(
                'label' => __('Collapsed map label', 'ct_theme'),
                'default' => 'show map',
                'type' => 'input',
            ),
        );
    }

    /**
     * Returns additional info about VC
     * @return ctVisualComposerInfo
     */
    public function getVisualComposerInfo()
    {
        return new ctVisualComposerInfo($this, array(
	        'icon' => 'fa-globe',
	        'description' => __( "Create a Google Map", 'ct_theme')
	        ));
    }
}

new ctGoogleMapsShortcode();