<?php

/**
 * Contact shortcode
 */
class ctDividerShortcode extends ctShortcode
{


    /**
     * Returns name
     * @return string|void
     */
    public function getName()
    {
        return 'Divider';
    }

    /**
     * Shortcode name
     * @return string
     */
    public function getShortcodeName()
    {
        return 'ct_divider';
    }

    /**
     * Handles shortcode
     * @param $atts
     * @param null $content
     * @return string
     */

    public function handle($atts, $content = null)
    {
        extract(shortcode_atts($this->extractShortcodeAttributes($atts), $atts));

        $bg = $default_type !== 'none' ? CT_THEME_ASSETS . '/images/' . $default_type : ($bg_image !== '' ? $bg_image : CT_THEME_ASSETS . '/images/' . $default_type);
        if ($style == 'narrow') {
            $contStart = '<div class="'.ct_sanitize_html_class(array('ct-hrContainer', $ct_u_padding_top, $ct_u_padding_bottom)).'">';
            $contEnd = '</div>';
        } else {
            $contStart = '';
            $contEnd = '';
        }

        $mainContainerAtts = array(
            'class' => array(
                'hr-custom',
                'ct-js-background',
                $class,
            ),
            'data-bg' => $bg,
        );

        if ($pattern_style == 'no-repeat') {
            $mainContainerAtts['data-bgrepeat'] = 'no-repeat';
        }
        return $contStart . '<hr' . $this->buildContainerAttributes($mainContainerAtts, $atts) . '>' . $contEnd;
    }


    /**
     * Returns config
     *
     * @return null
     */
    public function getAttributes()
    {
        return array(
            'default_type' => array(
                'label' => __('Built-in style', 'ct_theme'),
                'default' => 'hr2.png',
                'type' => 'select',
                'options' => array(
                    'hr1.png' => __('type 1', 'ct_theme'),
                    'hr2.png' => __('type 2', 'ct_theme'),
                    'hr3.png' => __('type 3', 'ct_theme'),
                    'none' => __('none', 'ct_theme'),
                )
            ),

            'style' => array(
                'label' => __('Style', 'ct_theme'),
                'default' => 'normal',
                'type' => 'select',
                'options' => array(
                    'normal' => __('normal', 'ct_theme'),
                    'narrow' => __('narrow', 'ct_theme'),
                )
            ),


            'bg_image' => array('label' => __("Divider pattern url", 'ct_theme'), 'default' => '', 'type' => 'image', 'help' => __("Image pattern", 'ct_theme'), 'dependency' => array(
                'element' => 'default_type',
                'value' => array('none'))),

            'pattern_style' => array('label' => __('pattern style', 'ct_theme'), 'default' => 'no-repeat', 'type' => "select",
                'options' => array(
                    'repeat' => __('repeat', 'ct_theme'),
                    'no-repeat' => __('no repeat', 'ct_theme'),)
            ),


            'ct_u_padding_top' => array(
                'label' => __('Padding top', 'ct_theme'),
                'default' => 'ct-u-paddingTop30',
                'type' => 'select',
                'choices' =>
                    array(
                        "" => __("none", "ct_theme"),
                        "ct-u-paddingTop5" => __("top 5", "ct_theme"),
                        "ct-u-paddingTop10" => __("top 10", "ct_theme"),
                        "ct-u-paddingTop15" => __("top 15", "ct_theme"),
                        "ct-u-paddingTop20" => __("top 20", "ct_theme"),
                        "ct-u-paddingTop30" => __("top 30", "ct_theme"),
                        "ct-u-paddingTop40" => __("top 40", "ct_theme"),
                        "ct-u-paddingTop50" => __("top 50", "ct_theme"),
                        "ct-u-paddingTop60" => __("top 60", "ct_theme"),
                        "ct-u-paddingTop70" => __("top 70", "ct_theme"),
                        "ct-u-paddingTop80" => __("top 80", "ct_theme"),
                        "ct-u-paddingTop90" => __("top 90", "ct_theme"),
                        "ct-u-paddingTop100" => __("top 100", "ct_theme"),
                        "ct-u-paddingTop150" => __("top 150", "ct_theme"),
                    ),
                'help' => sprintf(__('Add paddings to the element', 'ct_theme')),
                'supported_by' => array(),
                'not_supported_by' => array('icon'),
                'dependency' => array(
                    'element' => 'style',
                    'value' => array('narrow')
                )),

            'ct_u_padding_bottom' => array(
                'label' => __('Padding bottom', 'ct_theme'),
                'default' => 'ct-u-paddingBottom30',
                'type' => 'select',
                'choices' =>
                    array(
                        "" => __("none", "ct_theme"),
                        "ct-u-paddingBottom5" => __("bottom 5", "ct_theme"),
                        "ct-u-paddingBottom10" => __("bottom 10", "ct_theme"),
                        "ct-u-paddingBottom15" => __("bottom 15", "ct_theme"),
                        "ct-u-paddingBottom20" => __("bottom 20", "ct_theme"),
                        "ct-u-paddingBottom30" => __("bottom 30", "ct_theme"),
                        "ct-u-paddingBottom40" => __("bottom 40", "ct_theme"),
                        "ct-u-paddingBottom50" => __("bottom 50", "ct_theme"),
                        "ct-u-paddingBottom60" => __("bottom 60", "ct_theme"),
                        "ct-u-paddingBottom70" => __("bottom 70", "ct_theme"),
                        "ct-u-paddingBottom80" => __("bottom 80", "ct_theme"),
                        "ct-u-paddingBottom90" => __("bottom 90", "ct_theme"),
                        "ct-u-paddingBottom100" => __("bottom 100", "ct_theme"),
                        "ct-u-paddingBottom150" => __("bottom 150", "ct_theme"),
                    ),
                'help' => sprintf(__('Add paddings to the element', 'ct_theme')),
                'supported_by' => array(),
                'not_supported_by' => array('icon'),
                'dependency' => array(
                    'element' => 'style',
                    'value' => array('narrow')
                )),
            

            'class' => array('label' => __("Divider custom class", 'ct_theme'), 'default' => '', 'type' => 'input',
                'help' => __('Adding custom class allows you to set diverse styles in css to the element. Type in name of class, which you defined in css. You can add as much classes as you like.', 'ct_theme')),
        );
    }
}

new ctDividerShortcode();