<?php

/**
 * Image shortcode
 */
class ctImageShortcode extends ctShortcode implements ctVisualComposerShortcodeInterface
{

    /**
     * default image link base
     */
    const DEFAULT_IMG_SRC = "http://dummyimage.com/";

    /**
     * default image width
     */
    const DEFAULT_IMG_WIDTH = 90;

    /**
     * default image heightd
     */
    const DEFAULT_IMG_HEIGHT = 90;

    /**
     * Returns name
     * @return string|void
     */
    public function getName()
    {
        return 'Image';
    }

    /**
     * Shortcode name
     * @return string
     */
    public function getShortcodeName()
    {
        return 'img';
    }

    /**
     * Handles shortcode
     * @param $atts
     * @param null $content
     * @return string
     */

    public function handle($atts, $content = null)
    {

        extract(shortcode_atts($this->extractShortcodeAttributes($atts), $atts));

        $src = $src ? $src : $this->getDefaultImgSrc($width, $height);
        switch ($align) {
            case 'center':
                $alignClass = 'text-center';
                break;
            case 'left':
                $alignClass = 'text-left';
                break;
            case 'right':
                $alignClass = 'text-right';
                break;
            default:
                $alignClass='';

        }



        $mainContainerAtts = array(
            'class' => array(
                $class,
                $alignClass,
                $margin,
                $shadow
            ),
            'width' => $width ? (int)$width : '',
            'height' => $height ? (int)$height : '',
            'alt' =>  __('image', 'ct_theme'),
            'src' => $src,
        );


        if ($width) {
            $mainContainerAtts['width'] = $width ?  (int)$width . '"' : '';
        }
        if ($height) {
            $mainContainerAtts['height'] = $height ?  (int)$height . '"' : '';
        }


        $img = '<img' . $this->buildContainerAttributes($mainContainerAtts, $atts, 'imgAtts') . '>';

        //link
        if ($link) {
            $img = '<a href="' . esc_url($link) . '">' . $img . '</a>';
        }
        return do_shortcode($img);
    }

    /**
     * Returns config
     * @return null
     */
    public function getAttributes()
    {
        return array(
            'widgetmode' => array('default' => 'false', 'type' => false),
            'src' => array('label' => __("image", 'ct_theme'), 'default' => '', 'type' => 'image', 'help' => __("Image source", 'ct_theme')),
            'alt' => array('label' => __('alt', 'ct_theme'), 'default' => ' ', 'type' => 'input', 'help' => __("Alternate text", 'ct_theme')),
            'shadow'            => array(
                'label'   => __( "Add Shadow to Image", 'ct_theme' ),
                'type'    => 'checkbox',
                'default' => '',
                'value'   => 'ct-frame-imageShadow',
            ),
            'width' => array('label' => __('width', 'ct_theme'), 'default' => '', 'type' => 'input', 'help' => __("Image width", 'ct_theme')),
            'height' => array('label' => __('height', 'ct_theme'), 'default' => '', 'type' => 'input', 'help' => __("Image height", 'ct_theme')),
            'align' => array('label' => __('align', 'ct_theme'), 'default' => 'auto', 'type' => 'select', 'options' => array('default' => __('Default', 'ct_theme'), 'left' => __('Left', 'ct_theme'), 'center' => __('Center', 'ct_theme'), 'right' => __('Right', 'ct_theme')), 'help' => __("Image align", 'ct_theme')),

            'margin' => array('label' => __('Margin', 'ct_theme'), 'default' => 'auto', 'type' => 'select',
                'options' => array(
                    "" => __("none", "ct_theme"),
                    "ct-u-marginBoth10" => __("both 10", "ct_theme"),
                    "ct-u-marginTop10" => __("top 10", "ct_theme"),
                    "ct-u-marginBottom10" => __("bottom 10", "ct_theme"),
                    "ct-u-marginBoth20" => __("both 20", "ct_theme"),
                    "ct-u-marginTop20" => __("top 20", "ct_theme"),
                    "ct-u-marginBottom20" => __("bottom 20", "ct_theme"),
                    "ct-u-marginBoth30" => __("both 30", "ct_theme"),
                    "ct-u-marginTop30" => __("top 30", "ct_theme"),
                    "ct-u-marginBottom30" => __("bottom 30", "ct_theme"),
                    "ct-u-marginBoth40" => __("both 40", "ct_theme"),
                    "ct-u-marginTop40" => __("top 40", "ct_theme"),
                    "ct-u-marginBottom40" => __("bottom 40", "ct_theme"),
                    "ct-u-marginBoth50" => __("both 50", "ct_theme"),
                    "ct-u-marginTop50" => __("top 50", "ct_theme"),
                    "ct-u-marginBottom50" => __("bottom 50", "ct_theme"),
                    "ct-u-marginBoth60" => __("both 60", "ct_theme"),
                    "ct-u-marginTop60" => __("top 60", "ct_theme"),
                    "ct-u-marginBottom60" => __("bottom 60", "ct_theme"),
                    "ct-u-marginBoth70" => __("both 70", "ct_theme"),
                    "ct-u-marginTop70" => __("top 70", "ct_theme"),
                    "ct-u-marginBottom70" => __("bottom 70", "ct_theme"),
                    "ct-u-marginBoth80" => __("both 80", "ct_theme"),
                    "ct-u-marginTop80" => __("top 80", "ct_theme"),
                    "ct-u-marginBottom80" => __("bottom 80", "ct_theme"),
                    "ct-u-marginBoth90" => __("both 90", "ct_theme"),
                    "ct-u-marginTop90" => __("top 90", "ct_theme"),
                    "ct-u-marginBottom90" => __("bottom 90", "ct_theme"),
                    "ct-u-marginBoth100" => __("both 100", "ct_theme"),
                    "ct-u-marginTop100" => __("top 100", "ct_theme"),
                    "ct-u-marginBottom100" => __("bottom 100", "ct_theme"),
                    "ct-u-marginBoth150" => __("both 150", "ct_theme"),
                    "ct-u-marginTop150" => __("top 150", "ct_theme"),
                    "ct-u-marginBottom150" => __("bottom 150", "ct_theme"),
                ),


                'help' => __("Image align", 'ct_theme')),

            'link' => array('label' => __('link', 'ct_theme'), 'help' => __("ex. http://www.google.com", 'ct_theme')),
            'class' => array('label' => __("Custom class", 'ct_theme'), 'default' => '', 'type' => 'input', 'help' => __('Adding custom class allows you to set diverse styles in css to the element. Type in name of class, which you defined in css. You can add as much classes as you like.', 'ct_theme')),
        );
    }

    /**
     * returns default image source
     * @param $width
     * @param $height
     * @return string
     */
    protected function getDefaultImgSrc($width, $height)
    {
        if ($width && $height) {
            return self::DEFAULT_IMG_SRC . $width . "x" . $height;
        }
        return '';
    }

    /**
     * Returns additional info about VC
     * @return ctVisualComposerInfo
     */
    public function getVisualComposerInfo()
    {
        return new ctVisualComposerInfo($this, array(
            'icon' => 'fa-picture-o',
            'description' => __( "Add an image", 'ct_theme')
        ));
    }
}

new ctImageShortcode();