<?php

/**
 * ctChapter shortcode
 */
class ctSectionShortcode extends ctShortcode implements ctVisualComposerShortcodeInterface
{

    public function __construct()
    {
        parent::__construct(); // TODO: Change the autogenerated stub

        add_filter('ct.utility_class.add_custom_attributes.' . $this->getShortcodeName(), array(
            $this,
            'addUtilityClassAttribute'
        ), 10, 2);
    }


    /**
     * Add custom attribute
     *
     * @param $data
     * @param $shortcode
     */

    public function addUtilityClassAttribute($data, $shortcode)
    {
        //we will add a default background if we are in dark flavour
        if (isset($data['class'])) {
            if (in_array('inherit', $data['class']) && ct_get_context_option('general_flavour') == 'ct--darkMotive') {
            }
        }

        return $data;
    }

    /**
     * Returns name
     * @return string|void
     */
    public function getName()
    {
        return 'Section';
    }

    /**
     * Shortcode name
     * @return string
     */
    public function getShortcodeName()
    {
        return 'section';
    }

    /**
     * Returns shortcode type
     * @return mixed|string
     */

    public function getShortcodeType()
    {
        return self::TYPE_SHORTCODE_ENCLOSING;
    }

    private $context = '';



    public function enqueueScripts()
    {


    }


    /**
     * Handles shortcode
     *
     * @param $atts
     * @param null $content
     *
     * @return string
     */

    public function handle($atts, $content = null)
    {
        $atts = (shortcode_atts($this->extractShortcodeAttributes($atts), $atts));
        extract($atts);

        if (!isset($atts) || !is_array($atts) || empty($atts)) {
            return '';
        }


        $mainContainerAtts = array(
            'class' => array('ct-frame', $frame=='yes'?'ct-frame--white':'', $center),
            'id' => $atts['section_id']);

        return do_shortcode('<!--Row Start-->
				<section' . $this->buildContainerAttributes($mainContainerAtts, $atts) . '>
				 <div class="section">
				' . $content . '
				</div>
				</section>
			<!--Row End-->');


    }

    /**
     * Returns config
     * @return null
     */
    public function getAttributes()
    {

        return array(

            'frame' => array(
                'label' => __('Show frame?', 'ct_theme'),
                'default' => 'yes',
                'type' => 'select',
                'options' => array(
                    'yes' => 'yes',
                    'no' => 'no',
                )
            ),
            'section_id' => array('label' => __('ID', 'ct_theme'), 'default' => '', 'type' => 'input'),


            'content' => array(
                'label' => __('content', 'ct_theme'),
                'default' => '',
                'type' => "textarea"
            ),

            'center' => array(
                'label' => __('Center content?', 'ct_theme'),
                'default' => 'no',
                'type' => 'select',
                'options' => array(
                    '' => 'no',
                    'text-center' => 'yes'
                )
            ),
        );


    }

    /**
     * Returns additional info about VC
     * @return ctVisualComposerInfo
     */
    public function getVisualComposerInfo()
    {
        return new ctVisualComposerInfo($this, array('icon' => 'fa-cogs', 'container' => true));
    }
}

new ctSectionShortcode();