<?php

/**
 * Recent posts
 * ver. 2.1
 */
class ctRecentPostsShortcode extends ctShortcodeQueryable
{

    /**
     * Returns name
     * @return string|void
     */
    public function getName()
    {
        return 'Recent posts';
    }

    /**
     * Shortcode name
     * @return string
     */
    public function getShortcodeName()
    {
        return 'recent_posts';
    }


    /**
     * Handles shortcode
     * @param $atts
     * @param null $content
     * @return string
     */
    public function handle($atts, $content = null)
    {
        $attributes = shortcode_atts($this->extractShortcodeAttributes($atts), $atts);

        extract($attributes);
        $recentposts = $this->getCollection($attributes);
        $dividerRel = $this->buildRelatedShortcode('ct_divider', $atts, 'divider_rel');

        $mainContainerAtts = array(
            'class' => array(
                'ct-frame', 'ct-frame--white',
                $class),
        );


        $output = '
            <section' . $this->buildContainerAttributes($mainContainerAtts, $atts) . '>
            <h3 class="ct-u-font1 text-uppercase text-center">' . $header . '</h3>
            ' . $dividerRel;


        foreach ($recentposts as $p) {
            //Get the excerpt without cutting off the last word

            $postImage = ct_get_image_link(
                array(
                    'img_src' => ct_get_feature_image_src($p->ID, 'ct_portfolio_slider_thumb'),
                    'img_alt' => 'Blog image alt',
                    'link_url' => get_permalink($p->ID),
                    'link_class' => 'ct-js-popup ct-frame-image',
                    'link_attrs' => array(
                        array('attr' => 'title', 'val' => $p->post_title),
                    )
                ));

            $output .= $postImage;
            $output .= '<article class="ct-blogItem ct-formatPhoto ct-u-paddingBottom20">
    <div class="ct-innerMargin">
        <div class="ct-entryMeta">
            <div class="ct-entryDateFirst">' . get_the_date('', $p->ID) . '</div>
        </div>
        <h3 class="ct-entryTitle ct-u-font2"><a href="' . get_permalink($p->ID) . '">' . $p->post_title . '</a></h3>
        <div class="ct-entryDescription">' . ct_get_excerpt_stripped_by_id($p->ID) . '</div>
        <div class="clearfix"></div>
    </div>
</article>';
        }
        $output .= '</section>';
        return do_shortcode($output);
    }


    /**
     * Shortcode type
     * @return string
     */
    public function getShortcodeType()
    {
        return self::TYPE_SHORTCODE_SELF_CLOSING;
    }

    /**
     * Returns config
     * @return null
     */
    public function getAttributes()
    {
        $items = $this->getAttributesWithQuery(array(
            'header' => array('label' => __('Header', 'ct_theme'), 'default' => '', 'type' => 'input'),
            'limit' => array('label' => __('limit', 'ct_theme'), 'default' => 10, 'type' => 'input'),
            'class' => array('label' => __("Custom class", 'ct_theme'), 'default' => '', 'type' => 'input')
        ));
        $this->mergeShortcodeAttributes($items, 'ct_divider', 'divider_rel', __('Header divider', 'ct_theme'));

        return $items;
    }
}

new ctRecentPostsShortcode();