<?php
/**
 * Helper functions for theme
 */


add_filter('ct_customizer.style_names', 'extend_styles');
function extend_styles()
{
    $stylesNames = array('style', 'style2', 'style3', 'style4');
    return $stylesNames;
}

/**
 * Enqueue scripts
 */
if (!function_exists('ct_theme_scripts')) {
    /**
     * Load additional scripts per item
     */
    function ct_theme_scripts()
    {

        //include google font
        $googleFont = get_theme_mod('ct_customizer_navbar_family');
        if($googleFont == '' || is_null($googleFont)){
            $googleFont = 'Roboto Condensed';
        }
        if ('' !== $googleFont) {
            wp_register_style('customizer_fonts_' . esc_attr(str_replace(' ', '+', $googleFont)), "//fonts.googleapis.com/css?family={$googleFont}");
            wp_enqueue_style('customizer_fonts_' . esc_attr(str_replace(' ', '+', $googleFont)));
        }


        if (is_archive() || is_page_template('page-masonry.php') || is_page_template('page-slider-masonry.php')) {


            wp_register_script('ct_isotope', CT_THEME_ASSETS . '/js/portfolio/jquery.isotope.min.js', array('jquery'), null, true);
            wp_enqueue_script('ct_isotope');
            wp_register_script('ct_infinitescroll', CT_THEME_ASSETS . '/js/portfolio/infinitescroll.min.js', array('jquery'), null, true);
            wp_enqueue_script('ct_infinitescroll');
            wp_register_script('ct_imagesloaded', CT_THEME_ASSETS . '/js/portfolio/imagesloaded.js', array('jquery'), null, true);
            wp_enqueue_script('ct_imagesloaded');
            wp_register_script('ct_init', CT_THEME_ASSETS . '/js/portfolio/init.js', array('jquery'), null, true);
            wp_enqueue_script('ct_init');
            wp_register_script('ct_magnific_popup', CT_THEME_ASSETS . '/plugins/magnific-popup/jquery.magnific-popup.min.js', array('jquery'), null, true);
            wp_enqueue_script('ct_magnific_popup');
            wp_register_script('ct_imgbox_init', CT_THEME_ASSETS . '/js/image-box/init.js', array('jquery'), null, true);
            wp_enqueue_script('ct_imgbox_init');
            wp_register_style('ct-magnific-popup-style', CT_THEME_ASSETS . '/js/jplayer/jplayer.css');
            wp_enqueue_style('ct-magnific-popup-style');
        }

        if (is_page_template('page-menu.php')) {

            wp_register_script('ct_onepager', CT_THEME_ASSETS . '/js/one-pager/jquery.pagescroller.lite.js', array('jquery'), null, true);
            wp_enqueue_script('ct_onepager');
        }


        //always loaded
        wp_register_script('ct_device', CT_THEME_ASSETS . '/js/device.min.js', array('jquery'), null, true);
        wp_enqueue_script('ct_device');
        wp_register_script('ct_snap', CT_THEME_ASSETS . '/js/snap.min.js', array('jquery'), null, true);
        wp_enqueue_script('ct_snap');
        wp_register_script('ct_select2', CT_THEME_ASSETS . '/js/select2/select2.js', array('jquery'), null, true);
        wp_enqueue_script('ct_select2');


        if (apply_filters('ct_theme_loader.load_styles', true)) {
            if (file_exists(CT_THEME_DIR . '/assets/css/style2.css')) {
                wp_enqueue_style('ct_theme2', CT_THEME_DIR_URI . '/assets/css/style2.css', array('ct_theme'), null);
            } elseif (file_exists(CT_THEME_DIR . '/assets/less/style2.less')) {
                wp_enqueue_style('ct_theme2', CT_THEME_DIR_URI . '/ct/css.php?file=css/style2.less', array('ct_theme'), null);
            }
            if (file_exists(CT_THEME_DIR . '/assets/css/style3.css')) {
                wp_enqueue_style('ct_theme3', CT_THEME_DIR_URI . '/assets/css/style3.css', array('ct_theme'), null);
            } elseif (file_exists(CT_THEME_DIR . '/assets/less/style3.less')) {
                wp_enqueue_style('ct_theme3', CT_THEME_DIR_URI . '/ct/css.php?file=css/style3.less', array('ct_theme'), null);
            }
            if (file_exists(CT_THEME_DIR . '/assets/css/style4.css')) {
                wp_enqueue_style('ct_theme4', CT_THEME_DIR_URI . '/assets/css/style4.css', array('ct_theme'), null);
            } elseif (file_exists(CT_THEME_DIR . '/assets/less/style4.less')) {
                wp_enqueue_style('ct_theme4', CT_THEME_DIR_URI . '/ct/css.php?file=css/style4.less', array('ct_theme'), null);
            }
        }

    }

}
add_action('wp_enqueue_scripts', 'ct_theme_scripts');


/**
 * @param $classes
 *
 * @return array
 */
function ct_custom_body_class($classes)
{
    $classes[] = 'cssAnimate';
    $ptn = ct_get_context_option('general_content_pattern', '13');
    if ((int)$ptn > 0 && (int)$ptn <= 13) {
        $classes[] = 'ptn' . $ptn;
    }

    if (is_page_template('page-contact.php')) {
        $classes[] = 'ct--pageVariation';
    }

    return $classes;
}

add_filter('body_class', 'ct_custom_body_class', 10, 2);


/**
 * @param $title
 * @return mixed|null|string|void
 */
function ct_title($title)
{
    if (is_single() || (is_home() && !is_front_page()) || (is_page() && !is_front_page())) {
        $title = single_post_title('', false);
    }
    if (is_front_page() && !is_page()) {
        $title = esc_attr(get_bloginfo('name'));
    }

    if (is_archive()) {
        $title = ct_get_title();
    }

    return $title;
}

add_filter('wp_title', 'ct_title');


function remove_menuclass($ulclass)
{
    return preg_replace('/<a class="dropdown-toggle" data-toggle="dropdown" data-target="#"/', '<a ', $ulclass, -1);
}

add_filter('wp_nav_menu', 'remove_menuclass');




