<?php
if (!class_exists('ctMenuType')) {
    //require_once CT_THEME_LIB_DIR . '/types/ctTypeBase.class.php';


    /**
     * Custom type - ctTestimonialType
     */
    class ctMenuType extends ctTypeBase
    {


        /**
         * Slug option name
         */

        const OPTION_SLUG = 'menu_index_slug';

        /**
         * Initializes Menu
         * @return mixed|void
         */

        public function init()

        {
            add_action('template_redirect', array($this, 'menuContextFixer'));
            $this->registerType();
            $this->registerTaxonomies();
            add_action("admin_init", array($this, "addMetaBox"));
        }

        /**
         * Adds meta box
         */

        public function addMetaBox()
        {
            add_meta_box("menu-meta", __("Menu settings", 'ct_theme'), array($this, "menuMeta"), "ct_menu", "normal", "high");

            add_action('save_post', array($this, 'saveDetails'));
        }

        /**
         * Fixes proper menu state
         */

        public function menuContextFixer()
        {
            if (get_query_var('post_type') == 'menu') {
                global $wp_query;
                $wp_query->is_home = false;
            }
            if (get_query_var('taxonomy') == 'menu_category') {
                global $wp_query;
                $wp_query->is_404 = true;
                $wp_query->is_tax = false;
                $wp_query->is_archive = false;
            }
        }

        /**
         * Register type
         */

        protected function registerType()
        {
            $typeData = $this->callFilter('pre_register_type', array(
                'labels' => array(
                    'name' => _x('Menu', 'post type general name', 'ct_theme'),
                    'singular_name' => _x('Menu item', 'post type singular name', 'ct_theme'),
                    'add_new' => _x('Add New menu item', 'ct_theme'),
                    'add_new_item' => __('Add New menu item', 'ct_theme'),
                    'edit_item' => __('Edit menu item', 'ct_theme'),
                    'new_item' => __('New menu item', 'ct_theme'),
                    'view_item' => __('View menu', 'ct_theme'),
                    'search_items' => __('Search menu items', 'ct_theme'),
                    'not_found' => __('No menu items found', 'ct_theme'),
                    'not_found_in_trash' => __('No menu items found in Trash', 'ct_theme'),
                    'parent_item_colon' => '',
                    'menu_name' => __('Restaurant', 'ct_theme'),
                ),
                'singular_label' => __('Menu item', 'ct_theme'),
                'public' => true,
                'publicly_queryable' => true,
                'exclude_from_search' => false,
                'show_ui' => true,
                'show_in_menu' => true,
                //'menu_position' => 20,
                'menu_icon' => 'dashicons-clipboard',
                'capability_type' => 'post',
                'hierarchical' => false,
                'supports' => array('title', 'editor', 'thumbnail', 'page-attributes'),
                'has_archive' => false,
                'rewrite' => array('slug' => $this->getPermalinkSlug(), 'with_front' => true, 'pages' => true, 'feeds' => false),
                'query_var' => false,
                'can_export' => true,
                'show_in_nav_menus' => true,
                'taxonomies' => array('post_tag')
            ));

            register_post_type('ct_menu', $typeData);
            $this->callHook('post_register_type');
        }

        /**
         * Returns permalink slug
         * @return string
         */

        protected function getPermalinkSlug()
        {
            // Rewriting Permalink Slug
            if (function_exists('ct_get_option')) {
                $permalink_slug = ct_get_option('th', 'menu');
            } else {
                $permalink_slug = 'menu';
            }

            if (empty($permalink_slug)) {
                $permalink_slug = 'menu';
            }

            return $permalink_slug;
        }

        /**
         * Gets hook name
         * @return string
         */
        protected function getHookBaseName()
        {
            return 'ct_menu';
        }

        /**
         * Creates taxonomies
         */

        protected function registerTaxonomies()
        {
            $data = $this->callFilter('pre_register_taxonomies', array(
                'hierarchical' => true,
                'labels' => array(
                    'name' => _x('Menu Categories', 'taxonomy general name', 'ct_theme'),
                    'singular_name' => _x('Menu Category', 'taxonomy singular name', 'ct_theme'),
                    'search_items' => __('Search Categories', 'ct_theme'),
                    'popular_items' => __('Popular Categories', 'ct_theme'),
                    'all_items' => __('All Categories', 'ct_theme'),
                    'parent_item' => null,
                    'parent_item_colon' => null,
                    'edit_item' => __('Edit Menu Category', 'ct_theme'),
                    'update_item' => __('Update Menu Category', 'ct_theme'),
                    'add_new_item' => __('Add new Menu Category', 'ct_theme'),
                    'new_item_name' => __('New Menu Category Name', 'ct_theme'),
                    'separate_items_with_commas' => __('Separate Menu category with commas', 'ct_theme'),
                    'add_or_remove_items' => __('Add or remove Menu category', 'ct_theme'),
                    'choose_from_most_used' => __('Choose from the most used Menu category', 'ct_theme'),
                    'menu_name' => __('Categories', 'ct_theme'),
                ),
                'public' => false,
                'show_in_nav_menus' => false,
                'show_ui' => true,
                'show_tagcloud' => false,
                'query_var' => 'ct_menu_category',
                'rewrite' => false,

            ));
            register_taxonomy('ct_menu_category', 'ct_menu', $data);
            $this->callHook('post_register_taxonomies');
        }


        /**
         * Handles rebuild
         */

        public function handleSlugOptionSaved($newValues)
        {
            $currentSlug = $this->getPermalinkSlug();
            //rebuild rewrite if new slug
            if (isset($newValues[self::OPTION_SLUG]) && ($currentSlug != $newValues[self::OPTION_SLUG])) {
                $this->callHook('pre_slug_option_saved', array('current_slug' => $currentSlug, 'new_slug' => $newValues[self::OPTION_SLUG]));

                //clean rewrite to refresh it
                delete_option('rewrite_rules');
            }
        }


        /**
         * Draw s Testimonial meta
         */

        public function menuMeta()
        {
            global $post;
            $custom = get_post_custom($post->ID);


            $price = isset($custom["ct_price"][0]) ? $custom["ct_price"][0] : "";
            ?>

            <p>
                <label
                    for="ct_price"><?php _e("Price", 'ct_theme') ?> </label>
                <input id="ct_price" class="regular-text" name="ct_price"
                       value="<?php echo esc_attr($price); ?>"/>
            </p>

            <?php
        }


        public function saveDetails()
        {
            global $post;

            $fields = array(
                'ct_price',
            );

            if (class_exists('ctSocialsMetaFields')) {
                $fields = array_merge(ctSocialsMetaFields::getFieldsArray(), $fields);
            }

            foreach ($fields as $field) {
                if (isset($_POST[$field])) {
                    update_post_meta($post->ID, $field, $_POST[$field]);
                }
            }
        }
    }

    new ctMenuType();
}