<?php
if (!class_exists('ctSidebarBlockType')) {
    //require_once CT_THEME_LIB_DIR . '/types/ctTypeBase.class.php';

    /**
     * Custom type - ctTestimonialType
     */
    class ctSidebarBlockType extends ctTypeBase
    {


        /**
         * Slug option name
         */

        const OPTION_SLUG = 'team_index_slug';

        /**
         * Initializes Team
         * @return mixed|void
         */

        public function init()

        {
            add_action('template_redirect', array($this, 'teamContextFixer'));
            $this->registerType();
            $this->registerTaxonomies();
            add_action("admin_init", array($this, "addMetaBox"));
        }

        /**
         * Adds meta box
         */

        public function addMetaBox()
        {
            add_meta_box("team-meta", __("Team settings", 'ct_theme'), array($this, "blockMeta"), "team", "normal", "high");
            add_action('save_post', array($this, 'saveDetails'));
        }

        /**
         * Fixes proper menu state
         */

        public function teamContextFixer()
        {
            if (get_query_var('post_type') == 'ct_sidebar_block') {
                global $wp_query;
                $wp_query->is_home = false;
            }
            if (get_query_var('taxonomy') == 'sidebar_block_category') {
                global $wp_query;
                $wp_query->is_404 = true;
                $wp_query->is_tax = false;
                $wp_query->is_archive = false;
            }
        }

        /**
         * Register type
         */

        protected function registerType()
        {
            $typeData = $this->callFilter('pre_register_type', array(
                'labels' => array(
                    'name' => _x('Sidebar Blocks', 'post type general name', 'ct_theme'),
                    'singular_name' => _x('Sidebar Block', 'post type singular name', 'ct_theme'),
                    'add_new' => _x('Add New', 'Sidebar Block', 'ct_theme'),
                    'add_new_item' => __('Add New sidebar block', 'ct_theme'),
                    'edit_item' => __('Edit sidebar block', 'ct_theme'),
                    'new_item' => __('New sidebar block', 'ct_theme'),
                    'view_item' => __('View sidebar block', 'ct_theme'),
                    'search_items' => __('Search sidebar blocks', 'ct_theme'),
                    'not_found' => __('No sidebar blocks found', 'ct_theme'),
                    'not_found_in_trash' => __('No sidebar blocks found in Trash', 'ct_theme'),
                    'parent_item_colon' => '',
                    'menu_name' => __('Sidebar Blocks', 'ct_theme'),
                ),
                'singular_label' => __('Sidebar block', 'ct_theme'),
                'public' => true,
                'publicly_queryable' => true,
                'exclude_from_search' => false,
                'show_ui' => true,
                'show_in_menu' => true,
                //'menu_position' => 20,
                'menu_icon' => 'dashicons-media-code',
                'capability_type' => 'post',
                'hierarchical' => false,
                'supports' => array('title', 'editor', 'thumbnail', 'page-attributes'),
                'has_archive' => false,
                'rewrite' => array('slug' => $this->getPermalinkSlug(), 'with_front' => true, 'pages' => true, 'feeds' => false),
                'query_var' => false,
                'can_export' => true,
                'show_in_nav_menus' => true,
                'taxonomies' => array('post_tag')
            ));

            register_post_type('ct_sidebar_block', $typeData);
            $this->callHook('post_register_type');
        }

        /**
         * Returns permalink slug
         * @return string
         */

        protected function getPermalinkSlug()
        {
            // Rewriting Permalink Slug
            if (function_exists('ct_get_option')){
                $permalink_slug = ct_get_option('th', 'sidebar_block');
            }else{
                $permalink_slug ='sidebar_block';
            }

            if (empty($permalink_slug)) {
                $permalink_slug = 'sidebar_block';
            }

            return $permalink_slug;
        }

        /**
         * Gets hook name
         * @return string
         */
        protected function getHookBaseName()
        {
            return 'ct_sidebar_block';
        }

        /**
         * Creates taxonomies
         */

        protected function registerTaxonomies()
        {
            $data = $this->callFilter('pre_register_taxonomies', array(
                'hierarchical' => true,
                'labels' => array(
                    'name' => _x('Sidebar block categories', 'taxonomy general name', 'ct_theme'),
                    'singular_name' => _x('Sidebar block', 'taxonomy singular name', 'ct_theme'),
                    'search_items' => __('Search Categories', 'ct_theme'),
                    'popular_items' => __('Popular Categories', 'ct_theme'),
                    'all_items' => __('All Categories', 'ct_theme'),
                    'parent_item' => null,
                    'parent_item_colon' => null,
                    'edit_item' => __('Edit Sidebar block Category', 'ct_theme'),
                    'update_item' => __('Update Sidebar block Category', 'ct_theme'),
                    'add_new_item' => __('Add New Sidebar block Category', 'ct_theme'),
                    'new_item_name' => __('New Sidebar block Category Name', 'ct_theme'),
                    'separate_items_with_commas' => __('Separate Sidebar block category with commas', 'ct_theme'),
                    'add_or_remove_items' => __('Add or remove Sidebar block category', 'ct_theme'),
                    'choose_from_most_used' => __('Choose from the most used Sidebar block category', 'ct_theme'),
                    'menu_name' => __('Categories', 'ct_theme'),
                ),
                'public' => false,
                'show_in_nav_menus' => false,
                'show_ui' => true,
                'show_tagcloud' => false,
                'query_var' => 'sidebar_block_category',
                'rewrite' => false,

            ));
            register_taxonomy('sidebar_block_category', 'ct_sidebar_block', $data);
            $this->callHook('post_register_taxonomies');
        }


        /**
         * Handles rebuild
         */

        public function handleSlugOptionSaved($newValues)
        {
            $currentSlug = $this->getPermalinkSlug();
            //rebuild rewrite if new slug
            if (isset($newValues[self::OPTION_SLUG]) && ($currentSlug != $newValues[self::OPTION_SLUG])) {
                $this->callHook('pre_slug_option_saved', array('current_slug' => $currentSlug, 'new_slug' => $newValues[self::OPTION_SLUG]));

                //clean rewrite to refresh it
                delete_option('rewrite_rules');
            }
        }


        /**
         * Draw s Testimonial meta
         */

        public function blockMeta()
        {
            global $post;
            $custom = get_post_custom($post->ID);


           /* $team_position = isset($custom["team_position"][0]) ? $custom["team_position"][0] : "";
            $description = isset($custom["description"][0]) ? $custom["description"][0] : "";
            $team_single_motive_color = isset($custom["team_single_motive_color"][0]) ? $custom["team_single_motive_color"][0] : "purple";*/
            ?>




            <?php
        }


        public function socialsMeta()
        {

            global $post;
            $custom = get_post_custom($post->ID);
            $socials = new ctSocialsMetaFields();
            echo $socials->getTheFields($custom);

            //var_dump(ctSocialsMetaFields::getFieldsArray());
        }


        /**
         * testimonial_footer template settings
         */

        public function testimonialTemplateMeta()
        {
            global $post;
            $custom = get_post_custom($post->ID);

        }


        public function saveDetails()
        {
            global $post;

            $fields = array(

            );



            foreach ($fields as $field) {
                if (isset($_POST[$field])) {
                    update_post_meta($post->ID, $field, $_POST[$field]);
                }
            }
        }
    }

    new ctSidebarBlockType();
}